﻿
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;


namespace JayJayWon
{
    public class GeneralPointerEnterExit : MonoBehaviour, IPointerEnterHandler, IPointerExitHandler
    {
        private Action enterCallback;
        private Action exitCallback;

        public void Init(Action enterCallback, Action exitCallback)
        {
            this.enterCallback =enterCallback;
            this.exitCallback =exitCallback;
        }
        public void OnPointerEnter(PointerEventData eventData)
        {

            enterCallback();
        }

        public void OnPointerExit(PointerEventData eventData)
        {
            exitCallback();
        }
    }

    public class GeneralPointerEnterExitHelp : MonoBehaviour, IPointerEnterHandler, IPointerExitHandler
    {
        private string helpTextOnEnter="";
        private string helpTextOnExit="";
        private JSONStorableString helpTextJSS;
        private bool warning = false;
        public void Init(string helpTextOnEnter, JSONStorableString _helpTextJSS, bool warning, string helpTextOnExit = "")
        {
            helpTextJSS = _helpTextJSS;
            this.helpTextOnEnter = helpTextOnEnter;
            this.helpTextOnExit = helpTextOnExit;
            this.warning = warning;
        }
        public void OnPointerEnter(PointerEventData eventData)
        {

            helpTextJSS.val = helpTextOnEnter;

            if (helpTextJSS.text != null)
            {
                Image image = helpTextJSS.text.transform.parent.gameObject.GetComponent<Image>();
                if (image != null)
                {
                    if (warning) image.color = Color.red;
                    else image.color = Color.gray;
                }
            }
        }

        public void OnPointerExit(PointerEventData eventData)
        {
            helpTextJSS.val = helpTextOnExit;

            if (helpTextJSS.text != null)
            {
                Image image = helpTextJSS.text.transform.parent.gameObject.GetComponent<Image>();
                if (image != null) image.color = Color.gray;
                
            }
        }
    }

    public class CustomUIElement
    {
        protected UIDynamic _spacerLeft { get; set; }
        protected UIDynamic _spacerRight { get; set; }
        public UIDynamic _spacerActive { get; protected set; }

        protected float spacerHeight;
        public float height { get { return spacerHeight; } }
        protected HashSet<UIDynamicPopup> uidPopuplist;


        public CustomUIElement(float height)
        {
            spacerHeight = height;
            uidPopuplist = new HashSet<UIDynamicPopup>();
        }

        public void CloseAllPopups(UIDynamicPopup exception)
        {
            foreach (UIDynamicPopup popup in uidPopuplist)
            {
                if (popup != exception) popup.popup.visible = false;
            }
        }

        public virtual void ClearUI()
        {
            if (_spacerLeft != null) UIASceneButtonsSetup.mvrScript.RemoveSpacer(_spacerLeft);
            if (_spacerRight != null) UIASceneButtonsSetup.mvrScript.RemoveSpacer(_spacerRight);
            _spacerActive = null;
            uidPopuplist.Clear();
        }

        public GameObject CreateVScrollableContent(int anchorPreset, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, float scrollBarWidth = 10f, float spacing = 0f)
        {
            var gameObject = UI.CreateVScrollableContent(_spacerActive.gameObject, new Color(1f,1f,1f,0.2f), anchorPreset, horizontalSize, verticalSize, anchoredPositionOffset,scrollBarWidth,spacing);
            return gameObject;
        }
        
        protected UIDynamicPopup InstantiatePopup(JSONStorableStringChooser jssc, bool scrollable)
        {
            Transform originalTransform = UIASceneButtonsSetup.mvrScript.manager.configurableScrollablePopupPrefab.transform;
            if (!scrollable) originalTransform = UIASceneButtonsSetup.mvrScript.manager.configurablePopupPrefab.transform;

            Transform popup = UnityEngine.Object.Instantiate(originalTransform, _spacerActive.gameObject.transform, false);
            UIDynamicPopup selectTargetPopup = popup.GetComponent<UIDynamicPopup>();

            selectTargetPopup.popup.showSlider = false;
            popup.SetParent(_spacerActive.gameObject.transform, false);
            jssc.popup = selectTargetPopup.popup;
            return (selectTargetPopup);
        }
        protected UIDynamicSlider InstantiateSlider(JSONStorableFloat jsf)
        {
            Transform originalTransform = UIASceneButtonsSetup.mvrScript.manager.configurableSliderPrefab.transform;

            Transform slider = UnityEngine.Object.Instantiate(originalTransform, _spacerActive.gameObject.transform, false);
            UIDynamicSlider newSlider = slider.GetComponent<UIDynamicSlider>();

            slider.SetParent(_spacerActive.gameObject.transform, false);
            jsf.slider = newSlider.slider;
            return (newSlider);
        }

        private UIDynamicSlider CreateSlider(JSONStorableFloat jsf, float width, float height, float xOffsetPosition, float yOffsetPostion = 0f)
        {
            RectTransform spacerRT = _spacerActive.GetComponent<RectTransform>();

            spacerRT.anchorMin = new Vector2(0f, 1f);
            spacerRT.anchorMax = new Vector2(0f, 1f);
            spacerRT.pivot = new Vector2(0f, 1f);

            UIDynamicSlider newSlider = InstantiateSlider(jsf);
            
            RectTransform childRect = newSlider.transform.GetComponent<RectTransform>();
            childRect.anchorMin = new Vector2(0f, 1f);
            childRect.anchorMax = new Vector2(0f, 1f);
            childRect.pivot = new Vector2(0f, 1f);

            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            childRect.transform.localPosition = new Vector3(xOffsetPosition, yOffsetPostion, 0f);

            return (newSlider);
        }

        // This prevents the popup panel from being rendered behind other items. Needs to be set at the end of the Frame to override the VAM default behaviour.
        protected IEnumerator ReparentPopup(UIDynamicPopup popupUID, bool forceRightParentPopup)
        {
            yield return new WaitForEndOfFrame();

            try
            {
                if (popupUID.popup.visible)
                {
                    if (forceRightParentPopup) popupUID.popup.popupPanel.transform.SetParent(_spacerRight.gameObject.transform.parent);
                    else popupUID.popup.popupPanel.transform.SetParent(_spacerActive.gameObject.transform.parent);
                }
            }
            catch (Exception e) { SuperController.LogError("BA.CustomUIElement.ReparentPopup: Exception caught:\n" + e); }
        }

        public UIDynamicPopup AddPopup(JSONStorableStringChooser jssc, UIPopup.OnOpenPopup callbackMethod, float width, float height, float xOffsetPosition, float yOffsetPosition = 0, float panelHeight = 0f, float popupRowHeight=50f,string labelOverride = null, float labelProportion = 0.4f)
        {
            UIDynamicPopup newPopup;
            if (panelHeight == 0f) newPopup = UI.CreatePopup(_spacerActive.gameObject, jssc, width, height, labelOverride, 30, xOffsetPosition, yOffsetPosition, anchorPreset: AnchorPresets.topLeft, popupRowHeight: popupRowHeight, labelProportion: labelProportion,rescaleText:false);
            else newPopup = UI.CreateScrollablePopup(_spacerActive.gameObject, jssc, width, height, labelOverride, 30, xOffsetPosition, yOffsetPosition, panelHeight, false, true, AnchorPresets.topLeft, labelProportion, popupRowHeight, 30, false,20);

            if (callbackMethod!=null) newPopup.popup.onOpenPopupHandlers += callbackMethod;

            newPopup.popup.onOpenPopupHandlers += () =>
            {
                //                UIAPluginUI.CloseAllPopups(newPopup);
                UIASceneButtonsSetup.mvrScript.StartCoroutine(ReparentPopup(newPopup, false));
            };

            newPopup.popup.selectColor = UI.UIhighlightColor;

            uidPopuplist.Add(newPopup);

            return newPopup;
        }

        public UIDynamicColorPicker AddColorPicker(JSONStorableColor jsc, float width, float height, float xOffsetPosition, float yOffsetPosition, string altLabel=null){
             UIDynamicColorPicker colorPicker= UI.CreateColorPicker(_spacerActive.gameObject,jsc,width,height,altLabel, xOffsetPosition,yOffsetPosition,AnchorPresets.topLeft);
            return colorPicker;

        }

        public UIDynamicToggle AddToggle (JSONStorableBool jsb, float width, float height, float xOffsetPosition, float yOffsetPosition, string labelOverride=null, float checkboxSize = 50f)
        {
            UIDynamicToggle newToggle= UI.CreateToggle(_spacerActive.gameObject,jsb,width,height, labelOverride, 30, xOffsetPosition,yOffsetPosition,AnchorPresets.topLeft, checkboxSize,false);
            return newToggle;
        }
        public UIDynamicTextField AddTextBox(JSONStorableString jss, float width, float height, float xOffsetPosition, float yOffsetPosition, TextAnchor textAlignment = TextAnchor.MiddleLeft, float vScrollBarWidth=10f)
        {

            UIDynamicTextField newTextField = UI.CreateTextBox(_spacerActive.gameObject, jss, width, height, 30, xOffsetPosition, yOffsetPosition, AnchorPresets.topLeft,textAlignment: textAlignment,rescaleText:false, vScrollBarWidth: vScrollBarWidth);
            return newTextField;
        }
        public UIDynamicTextField AddTextEntryBox(JSONStorableString jss, float width, float height, int fontSize, float xOffsetPosition, float yOffsetPosition, TextAnchor textAlignment = TextAnchor.MiddleLeft, InputField.LineType inputFieldType = InputField.LineType.MultiLineNewline, string placeHolderText = "", float hScrollBarHeight = 0f, float vScrollBarWidth = 10f, Action<string> textEntryChangeCallback = null, Action<string> textEntryEndCallback = null)
        {

            UIDynamicTextField newTextField = UI.CreateTextBox(_spacerActive.gameObject, jss, width, height, fontSize, xOffsetPosition, yOffsetPosition, AnchorPresets.topLeft,true,inputFieldType,placeHolderText,hScrollBarHeight,vScrollBarWidth, textEntryChangeCallback, textAlignment, textEntryEndCallback, false);
            return newTextField;
        }

        public UIDynamicTextField AddTextBox(string text, float width, float height, float xOffsetPosition, float yOffsetPosition,TextAnchor textAlignment = TextAnchor.MiddleLeft)
        {
            JSONStorableString jss = new JSONStorableString("textField", text);
            UIDynamicTextField newTextField = UI.CreateTextBox(_spacerActive.gameObject, jss, width, height, 30, xOffsetPosition, yOffsetPosition, AnchorPresets.topLeft,false, rescaleText: false, textAlignment: textAlignment);
            return newTextField;
        }



        public UIDynamicButton AddButton(float width, float height, float xOffsetPosition, float yOffsetPosition, string label, UnityEngine.Events.UnityAction callback)
        {
            UIDynamicButton newButton = UI.CreateUIButton(_spacerActive.gameObject,width,height,label, 30, xOffsetPosition,yOffsetPosition,AnchorPresets.topLeft,false, rescaleText:false);
            if (callback!=null) newButton.button.onClick.AddListener(callback);
            return newButton;
        }

        public UIDynamicSlider AddSlider(JSONStorableFloat jsf, float width, float height, float xOffsetPostion, float yOffsetPostion)
        {
            UIDynamicSlider newSlider = CreateSlider(jsf, width, height, xOffsetPostion, yOffsetPostion);
            newSlider.label = jsf.name;

            return newSlider;
        }

        protected UIDynamicButton InstantiateUIButton()
        {
            Transform buttonTransform = UnityEngine.Object.Instantiate(UIASceneButtonsSetup.mvrScript.manager.configurableButtonPrefab.transform, _spacerActive.gameObject.transform, false);
            UIDynamicButton selectTargetButton = buttonTransform.GetComponent<UIDynamicButton>();
            buttonTransform.SetParent(_spacerActive.gameObject.transform, false);
            return (selectTargetButton);
        }

        protected UIDynamicButton CreateButton(float width, float height, float xOffsetPosition, float yOffsetPostion = 0f)
        {
            RectTransform spacerRT = _spacerActive.GetComponent<RectTransform>();

            spacerRT.anchorMin = new Vector2(0f, 1f);
            spacerRT.anchorMax = new Vector2(0f, 1f);
            spacerRT.pivot = new Vector2(0f, 1f);

            UIDynamicButton targetButton = InstantiateUIButton();

            RectTransform childRect = targetButton.button.GetComponent<RectTransform>();
            childRect.anchorMin = new Vector2(0f, 1f);
            childRect.anchorMax = new Vector2(0f, 1f);
            childRect.pivot = new Vector2(0f,1f);

            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
            childRect.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            childRect.transform.localPosition = new Vector3(xOffsetPosition, yOffsetPostion, 0f);

            return (targetButton);
        }

        public void InitUI(int customRowWidth)
        {
            if (customRowWidth != CustomRowWidth.right)
            {
                _spacerLeft = UIASceneButtonsSetup.mvrScript.CreateSpacer(false);
                _spacerActive = _spacerLeft;
                _spacerLeft.height = spacerHeight;


            }
            if (customRowWidth != CustomRowWidth.left)
            {
                _spacerRight = UIASceneButtonsSetup.mvrScript.CreateSpacer(true);
                _spacerRight.height = spacerHeight;
            }
            if (customRowWidth == CustomRowWidth.right)
            {
                _spacerActive = _spacerRight;
            }
        }

    }


    public abstract class BaseUITab
    {
        protected JSONStorableString helpTextJSS = new JSONStorableString("helpText", "");
        protected string defaultHelpText = "";

        protected class UIGroup
        {
            public int groupID;
            public BaseUITab uiTab;

            public void RefreshGroupUIStringChange(string val)
            {
                uiTab.RefreshGroupUI(groupID);
            }
        }


        public void ApplyHelp(UIDynamicSlider sliderUID, string helpText)
        {
            ApplyHelp(sliderUID.slider.gameObject, helpText);
            ApplyHelp(sliderUID.sliderControl.gameObject, helpText);
            ApplyHelp(sliderUID.rangeAdjustGroup.gameObject, helpText);
            ApplyHelp(sliderUID.quickButtonsGroup.gameObject, helpText);
            ApplyHelp(sliderUID.labelText.gameObject, helpText);
            ApplyHelp(sliderUID.defaultButton.gameObject, helpText);

            var inputTrans = sliderUID.transform.Find("ValueInputField");
            if (inputTrans != null) ApplyHelp(inputTrans.gameObject, helpText);

            var panelTrans = sliderUID.transform.Find("Panel");
            if (panelTrans != null) ApplyHelp(panelTrans.gameObject, helpText);
        }

        public void ApplyHelp(UIDynamicColorPicker colorUID, string helpText)
        {
            ApplyHelp(colorUID.colorPicker.gameObject, helpText);
            ApplyHelp(colorUID.pickerContainer.gameObject, helpText);
            ApplyHelp(colorUID.labelText.gameObject, helpText);
            ApplyHelp(colorUID.gameObject, helpText);
        }

        public void ApplyHelp(UIDynamicPopup popupUID, string helpText)
        {
            ApplyHelp(popupUID.popup.gameObject, helpText);
            ApplyHelp(popupUID.popup.popupPanel.gameObject, helpText);
        }
        public void ApplyHelp(UIDynamicToggle toggleUID, string helpText)
        {
            ApplyHelp(toggleUID.toggle.gameObject, helpText);
        }
        public void ApplyHelp(UIDynamicTextField textFieldUID, string helpText)
        {
            GameObject gameObject = textFieldUID.gameObject;
            if (textFieldUID.transform.GetComponent<InputField>() != null)
            {
                gameObject = textFieldUID.transform.GetComponent<InputField>().gameObject;
            }
            ApplyHelp(gameObject, helpText);
        }
        public void ApplyHelp(UIDynamicButton buttonUID, string helpText, bool warning = false)
        {
            ApplyHelp(buttonUID.button.gameObject, helpText, warning);
        }

        public void ClearHelp()
        {
            helpTextJSS.val = "";
            Image image = helpTextJSS.text.transform.parent.gameObject.GetComponent<Image>();
            image.color = Color.gray;
        }

        public void ApplyHelp(GameObject gameObject, string helpText, bool warning = false)
        {
            GeneralPointerEnterExitHelp contextHelp = gameObject.AddComponent<GeneralPointerEnterExitHelp>();
            contextHelp.Init(helpText, helpTextJSS, warning, defaultHelpText);
        }

        public class UIDynamicElementLists
        {
            public HashSet<UIDynamic> UIDSpacersList;
            public HashSet<UIDynamicButton> UIDButtonsList;
            public HashSet<UIDynamicSlider> UIDSlidersList;
            public HashSet<UIDynamicPopup> UIDPopupList;
            public HashSet<UIDynamicColorPicker> UIDColorPickerList;
            public HashSet<UIDynamicTextField> UIDTextFieldList;
            public HashSet<UIDynamicToggle> UIDToggleList;
            public HashSet<CustomUIElement> UIDCustomElementList;
            public HashSet<BaseUITab> childUITabs;

            public UIDynamicElementLists()
            {
                UIDSpacersList = new HashSet<UIDynamic>();
                UIDButtonsList = new HashSet<UIDynamicButton>();
                UIDSlidersList = new HashSet<UIDynamicSlider>();
                UIDPopupList = new HashSet<UIDynamicPopup>();
                UIDColorPickerList = new HashSet<UIDynamicColorPicker>();
                UIDTextFieldList = new HashSet<UIDynamicTextField>();
                UIDToggleList = new HashSet<UIDynamicToggle>();
                UIDCustomElementList = new HashSet<CustomUIElement>();
                childUITabs = new HashSet<BaseUITab>();
            }

            public void ClearUIDs()
            {
                MVRScript mvrs = UIASceneButtonsSetup.mvrScript;
                foreach (UIDynamic spacer in UIDSpacersList) if (spacer != null) mvrs.RemoveSpacer(spacer);
                foreach (UIDynamicButton button in UIDButtonsList) if (button != null) mvrs.RemoveButton(button);
                foreach (UIDynamicSlider slider in UIDSlidersList) if (slider != null) mvrs.RemoveSlider(slider);
                foreach (UIDynamicPopup popup in UIDPopupList) if (popup != null) mvrs.RemovePopup(popup);
                foreach (UIDynamicColorPicker colorpicker in UIDColorPickerList) if (colorpicker != null) mvrs.RemoveColorPicker(colorpicker);
                foreach (UIDynamicTextField textfield in UIDTextFieldList) if (textfield != null) mvrs.RemoveTextField(textfield);
                foreach (UIDynamicToggle toggle in UIDToggleList) if (toggle != null) mvrs.RemoveToggle(toggle);
                foreach (CustomUIElement Element in UIDCustomElementList) Element.ClearUI();

                foreach (BaseUITab childUITab in childUITabs) childUITab.ClearAllGroupsUI();
                UIDSpacersList.Clear();
                UIDButtonsList.Clear();
                UIDSlidersList.Clear();
                UIDPopupList.Clear();
                UIDColorPickerList.Clear();
                UIDTextFieldList.Clear();
                UIDToggleList.Clear();
                UIDCustomElementList.Clear();
            }

            public int GetUIElementsCount()
            {
                int count = 0;
                foreach (UIDynamic spacer in UIDSpacersList) count += 1;
                foreach (UIDynamicButton button in UIDButtonsList) count += 1;
                foreach (UIDynamicSlider slider in UIDSlidersList) count += 1;
                foreach (UIDynamicPopup popup in UIDPopupList) count += 1;
                foreach (UIDynamicColorPicker colorpicker in UIDColorPickerList) count += 1;
                foreach (UIDynamicTextField textfield in UIDTextFieldList) count += 1;
                foreach (UIDynamicToggle toggle in UIDToggleList) count += 1;
                foreach (CustomUIElement customElement in UIDCustomElementList) count += 1;
                foreach (BaseUITab childUITab in childUITabs) count += 1;

                return count;
            }
            public void CloseAllPopup(UIDynamicPopup exception)
            {
                foreach (UIDynamicPopup UIDpopup in UIDPopupList)
                {
                    if (UIDpopup != exception) UIDpopup.popup.visible = false;
                }
                foreach (CustomUIElement customElement in UIDCustomElementList)
                {
                    customElement.CloseAllPopups(exception);
                }
                foreach (BaseUITab childUITab in childUITabs)
                {
                    childUITab.CloseAllPopups(exception);
                }
            }
        }

        private Dictionary<int, UIDynamicElementLists> uiElementGroups;
        private Dictionary<int, List<JSONStorableParam>> JSPsGroups;

        protected int uiGroupCount = 1;

        public BaseUITab()
        {
            uiElementGroups = new Dictionary<int, UIDynamicElementLists>();
            JSPsGroups = new Dictionary<int, List<JSONStorableParam>>();
        }
        protected void ForceUIRefreshOnValChange(JSONStorableParam jsp, int uiRefreshGroup, BaseUITab uiTab = null)
        {
            UIGroup groupLR = new UIGroup();
            groupLR.groupID = uiRefreshGroup;

            if (uiTab == null) groupLR.uiTab = this;
            else groupLR.uiTab = uiTab;

            switch (jsp.type)
            {
                case JSONStorable.Type.StringChooser:
                    RegisterJSSCRefreshUIonValChange(jsp as JSONStorableStringChooser, groupLR);
                    break;
                case JSONStorable.Type.String:
                    RegisterJSSRefreshUIonValChange(jsp as JSONStorableString, groupLR);
                    break;
                case JSONStorable.Type.Float:
                    RegisterJSFRefreshUIonValChange(jsp as JSONStorableFloat, groupLR);
                    break;
                case JSONStorable.Type.Bool:
                    RegisterJSBRefreshUIonValChange(jsp as JSONStorableBool, groupLR);
                    break;
                default:
                    SuperController.LogError("BA.BaseUITab.RegisterJSRefreshUIonValChange: unsupported JSONStorableParam type " + jsp.type.ToString());
                    break;
            }
        }
        private void RegisterJSSCRefreshUIonValChange(JSONStorableStringChooser jssc, UIGroup groupLR)
        {

            if (jssc.popup == null)
            {
                SuperController.LogError("BA.BaseUITab.RegisterJSSCRefreshUIonValChange - attempt to register JSSC with no associated UI Popup");
                return;
            }
            jssc.popup.onValueChangeHandlers += groupLR.RefreshGroupUIStringChange;
        }

        private void RegisterJSBRefreshUIonValChange(JSONStorableBool jsb, UIGroup groupLR)
        {
            if (jsb.toggle == null)
            {
                SuperController.LogError("BA.BaseUITab.RegisterJSBRefreshUIonValChange - attempt to register JSB with no associated UI Toggle");
                return;
            }
            jsb.toggle.onValueChanged.AddListener((value) => groupLR.RefreshGroupUIStringChange(""));
        }

        private void RegisterJSSRefreshUIonValChange(JSONStorableString jss, UIGroup groupLR)
        {
            if (jss.inputField == null)
            {
                SuperController.LogError("BA.BaseUITab.RegisterJSSRefreshUIonValChange - attempt to register JSS with no associated inputfield");
                return;
            }
            jss.inputField.onValueChanged.AddListener((text) => groupLR.RefreshGroupUIStringChange(text));

        }

        private void RegisterJSFRefreshUIonValChange(JSONStorableFloat jsf, UIGroup groupLR)
        {

            if (jsf.slider == null)
            {
                SuperController.LogError("BA.BaseUITab.RegisterJSFRefreshUIonValChange - attempt to register JSF with no associated Slider");
                return;
            }
            jsf.slider.onValueChanged.AddListener((floatVal) => groupLR.RefreshGroupUIStringChange(floatVal.ToString()));

        }

        protected abstract void CreateGroupUI(int groupID);

        public void RefreshGroupUI(int startGroupID)
        {

            CloseAllPopups(null);

            for (int groupID = startGroupID; groupID <= uiGroupCount; groupID++)
            {
                ClearGroupUI(groupID);
            }

            for (int groupID = startGroupID; groupID <= uiGroupCount; groupID++)
            {
                CreateGroupUI(groupID);
            }
        }

        private UIDynamicElementLists GetUIElementsFromGroup(int groupID, Dictionary<int, UIDynamicElementLists> uiElementsGroups)
        {
            UIDynamicElementLists uiElementGroup;

            if (uiElementsGroups.ContainsKey(groupID)) uiElementGroup = uiElementsGroups[groupID];
            else
            {
                uiElementGroup = new UIDynamicElementLists();
                uiElementsGroups.Add(groupID, uiElementGroup);
            }
            return (uiElementGroup);
        }

        protected void RegisterCustomUIElement(int groupID, CustomUIElement customElement)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);     

            uiElements.UIDCustomElementList.Add(customElement);
        }

        protected void RegisterChildUITab(int groupID, BaseUITab childUITab)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);
            uiElements.childUITabs.Add(childUITab);
        }

        protected UIDynamic CreateSpacer(int groupID, float height = 10f, bool rightSide = false)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);

            UIDynamic spacer = UIASceneButtonsSetup.mvrScript.CreateSpacer(rightSide);
            uiElements.UIDSpacersList.Add(spacer);
            spacer.height = height;
            return (spacer);
        }

        public void ClearAllGroupsUI()
        {
            CloseAllPopups(null);

            foreach (KeyValuePair<int, UIDynamicElementLists> kvp in uiElementGroups) ClearGroupUI(kvp.Key);

        }
        public virtual void ClearGroupUI(int groupID)
        {
            UIDynamicElementLists uiElements = GetUIElementsFromGroup(groupID, uiElementGroups);

            List<JSONStorableParam> jspListForGroup = null;
            uiElements.ClearUIDs();
            if (JSPsGroups.ContainsKey(groupID)) jspListForGroup = JSPsGroups[groupID];

            if (jspListForGroup != null)
            {
                jspListForGroup.Clear();
            }

        }
        public void CloseAllPopups(UIDynamicPopup exception)
        {
            foreach (var uiElements in uiElementGroups.Values.ToList()) uiElements.CloseAllPopup(exception);
        }

    }



    public static class UI
    {
        public static Color UIhighlightColor = Color.cyan;

        public static GameObject CreateHScrollableContent(GameObject parentGO, Color backgroundColor, int anchorPreset, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, float spacing =0f)
        {
            GameObject scrollableContentGO = AddChildGOImage(parentGO, backgroundColor, anchorPreset, horizontalSize, verticalSize, anchoredPositionOffset);

            GameObject buttonListViewportGO = CreateHScrollableContentViewPort(scrollableContentGO, horizontalSize - 10f, verticalSize, spacing);
            buttonListViewportGO.name = "scrollableContentViewportGO";

            GameObject buttonListScrollbarGO = CreateScrollBar(scrollableContentGO, horizontalSize, 10f, 0f, 0f, Scrollbar.Direction.LeftToRight);
            buttonListScrollbarGO.name = "scrollBar";

            ScrollRect scrollRect = scrollableContentGO.AddComponent<ScrollRect>();
            GameObject buttonListContentGO = buttonListViewportGO.transform.Find("hScrollableContentGO").gameObject;
            scrollRect.content = buttonListContentGO.GetComponent<RectTransform>();
            scrollRect.horizontal = true;
            scrollRect.vertical = false;

            scrollRect.movementType = ScrollRect.MovementType.Clamped;
            scrollRect.viewport = buttonListViewportGO.GetComponent<RectTransform>();
            scrollRect.horizontalScrollbar = buttonListScrollbarGO.GetComponent<Scrollbar>();
            scrollRect.horizontalScrollbarVisibility = ScrollRect.ScrollbarVisibility.AutoHideAndExpandViewport;

            return (scrollableContentGO);
        }


        public static GameObject CreateVScrollableContent(GameObject parentGO, Color backgroundColor, int anchorPreset, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, float scrollBarWidth = 10f, float spacing =0f)
        {
            GameObject scrollableContentGO = AddChildGOImage(parentGO, backgroundColor, anchorPreset, horizontalSize, verticalSize, anchoredPositionOffset);

            GameObject buttonListViewportGO = CreateVScrollableContentViewPort(scrollableContentGO, horizontalSize - scrollBarWidth, verticalSize, spacing);
            buttonListViewportGO.name = "scrollableContentViewportGO";

            GameObject buttonListScrollbarGO = CreateScrollBar(scrollableContentGO, scrollBarWidth, verticalSize, 0f, 0f, Scrollbar.Direction.BottomToTop);
            buttonListScrollbarGO.name = "scrollBar";

            ScrollRect scrollRect = scrollableContentGO.AddComponent<ScrollRect>();
            GameObject buttonListContentGO = buttonListViewportGO.transform.Find("vScrollableContentGO").gameObject;
            scrollRect.content = buttonListContentGO.GetComponent<RectTransform>();
            scrollRect.horizontal = false;
            scrollRect.vertical = true;

            scrollRect.movementType = ScrollRect.MovementType.Clamped;
            scrollRect.viewport = buttonListViewportGO.GetComponent<RectTransform>();
            scrollRect.verticalScrollbar = buttonListScrollbarGO.GetComponent<Scrollbar>();
            scrollRect.verticalScrollbarVisibility = ScrollRect.ScrollbarVisibility.AutoHideAndExpandViewport;

            return (scrollableContentGO);
        }

        private static GameObject CreateScrollBar(GameObject parentGO, float width, float height, float xAnchorOffset, float yAnchorOffset, Scrollbar.Direction scrollbarDirection)
        {
            Sprite backgroundSprite = Resources.FindObjectsOfTypeAll<Sprite>().First(sprite => sprite.name == "Background");
            Sprite uiSprite = Resources.FindObjectsOfTypeAll<Sprite>().First(sprite => sprite.name == "UISprite");

            int anchorPreset = AnchorPresets.vStretchRight;
            if (scrollbarDirection == Scrollbar.Direction.LeftToRight || scrollbarDirection == Scrollbar.Direction.RightToLeft) anchorPreset = AnchorPresets.hStretchBottom;

            GameObject scrollbarGO = AddChildGOImage(parentGO, new Color(1f, 1f, 1f), anchorPreset, width, height, Vector2.zero, backgroundSprite);
            Scrollbar scrollbar = scrollbarGO.AddComponent<Scrollbar>();
            scrollbar.SetDirection(scrollbarDirection, false);
            scrollbarGO.SetActive(true);

            GameObject scrollbarSlidingAreaGO = new GameObject();
            scrollbarSlidingAreaGO.AddComponent<RectTransform>();
            scrollbarSlidingAreaGO.transform.SetParent(scrollbarGO.transform, false);
            scrollbarSlidingAreaGO.SetActive(true);
            ResizeRecTransform(scrollbarSlidingAreaGO, width, height, 0f, 0f, anchorPreset);

            GameObject scrollbarHandleGO = AddChildGOImage(scrollbarSlidingAreaGO, new Color(1f, 1f, 1f), AnchorPresets.centre, 3f, 3f, Vector2.zero, uiSprite);

            scrollbar.targetGraphic = scrollbarHandleGO.GetComponent<UnityEngine.UI.Image>();
            scrollbar.handleRect = scrollbarHandleGO.GetComponent<RectTransform>();

            return (scrollbarGO);
        }

        private static GameObject CreateHScrollableContentViewPort(GameObject parentGO, float width, float height, float spacing = 0f)
        {
            GameObject scrollableContentViewportGO = AddChildGOImage(parentGO, new Color(1f, 0f, 0f, 0f), AnchorPresets.hStretchTop, width, height, Vector2.zero);
            //           Mask blvpMask = scrollableContentViewportGO.AddComponent<Mask>();
            //           blvpMask.showMaskGraphic = false;
            RectMask2D altMask = scrollableContentViewportGO.AddComponent<RectMask2D>();

            GameObject scrollableContentGO = AddChildGOImage(scrollableContentViewportGO, new Color(0f, 0f, 1f, 0f), AnchorPresets.hStretchTop, 200f, 0f, Vector2.zero, null, false);
            scrollableContentGO.name = "hScrollableContentGO";

            HorizontalLayoutGroup hlg = scrollableContentGO.AddComponent<HorizontalLayoutGroup>();
            hlg.childAlignment = TextAnchor.UpperLeft;
            hlg.childForceExpandHeight = true;
            hlg.childForceExpandWidth = true;
            hlg.childControlHeight = false;
            hlg.childControlWidth = false;
            hlg.spacing = spacing;

            ContentSizeFitter csf = scrollableContentGO.AddComponent<ContentSizeFitter>();
            csf.horizontalFit = ContentSizeFitter.FitMode.MinSize;
            csf.verticalFit = ContentSizeFitter.FitMode.MinSize;

            return (scrollableContentViewportGO);
        }


        private static GameObject CreateVScrollableContentViewPort(GameObject parentGO, float width, float height, float spacing = 0f)
        {
            GameObject scrollableContentViewportGO = AddChildGOImage(parentGO, new Color(1f, 0f, 0f,0f), AnchorPresets.vStretchLeft, width, height, Vector2.zero);
 //           Mask blvpMask = scrollableContentViewportGO.AddComponent<Mask>();
 //           blvpMask.showMaskGraphic = false;
            RectMask2D altMask = scrollableContentViewportGO.AddComponent<RectMask2D>();

            GameObject scrollableContentGO = AddChildGOImage(scrollableContentViewportGO, new Color(0f, 0f, 1f,0f), AnchorPresets.vStretchLeft, 0f, 200f, Vector2.zero, null, false);
            scrollableContentGO.name = "vScrollableContentGO";

            VerticalLayoutGroup vlg = scrollableContentGO.AddComponent<VerticalLayoutGroup>();
            vlg.childAlignment = TextAnchor.UpperLeft;
            vlg.childForceExpandHeight = true;
            vlg.childForceExpandWidth = true;
            vlg.childControlHeight = false;
            vlg.childControlWidth = false;
            vlg.spacing = spacing;

            ContentSizeFitter csf = scrollableContentGO.AddComponent<ContentSizeFitter>();
            csf.horizontalFit = ContentSizeFitter.FitMode.MinSize;
            csf.verticalFit = ContentSizeFitter.FitMode.MinSize;

            return (scrollableContentViewportGO);
        }

        public static void AddImageComponent(GameObject go, Color color, Sprite sprite = null, bool enabled = true)
        {
            Image image = go.AddComponent<Image>();
            image.color = color;
            
            if (sprite!=null) image.sprite = sprite;
            image.type = Image.Type.Sliced;
            image.enabled = enabled;
            
        }

        public static GameObject AddChildImage(Transform parentTransform, Color color, Vector2 anchorMin, Vector2 anchorMax, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, Vector2 pivot, Sprite sprite = null, bool enabled = true)
        {
            GameObject go = new GameObject();
            go.transform.SetParent(parentTransform, false);

            AddImageComponent(go, color, sprite, enabled);
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.sizeDelta = Vector2.zero;
            rt.anchorMin = anchorMin;
            rt.anchorMax = anchorMax;
            rt.pivot = pivot;
            if (verticalSize != 0f) rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, verticalSize);
            if (horizontalSize != 0f) rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, horizontalSize);

            rt.anchoredPosition = anchoredPositionOffset;

            return go;
        }

        public static GameObject AddChildImage(GameObject parentGO, Color color, Vector2 anchorMin, Vector2 anchorMax, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, Vector2 pivot, Sprite sprite = null, bool enabled = true)
        {
            return AddChildImage(parentGO.transform, color, anchorMin, anchorMax, horizontalSize, verticalSize, anchoredPositionOffset, pivot, sprite, enabled);
        }
        public static GameObject AddChildGOImage(GameObject parentGO, Color color, int anchorPreset, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, Sprite sprite = null, bool enabled = true)
        {
            return AddChildImage(parentGO.transform, color, AnchorPresets.GetAnchorMin(anchorPreset), AnchorPresets.GetAnchorMax(anchorPreset), horizontalSize, verticalSize, anchoredPositionOffset, AnchorPresets.GetPivot(anchorPreset), sprite, enabled);
        }

        public static GameObject AddChildImage(Transform parentTransform, Color color, int anchorPreset, float horizontalSize, float verticalSize, Vector2 anchoredPositionOffset, Sprite sprite = null, bool enabled = true)
        {
            return AddChildImage(parentTransform, color, AnchorPresets.GetAnchorMin(anchorPreset), AnchorPresets.GetAnchorMax(anchorPreset), horizontalSize, verticalSize, anchoredPositionOffset, AnchorPresets.GetPivot(anchorPreset), sprite, enabled);
        }

        public static GameObject CreateBasicTextBox(GameObject parentGO, string textString, float width, float height, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool resizeTextForBestFit = false, int minFontSize = 20, HorizontalWrapMode horizontalWrapMode = HorizontalWrapMode.Wrap, Color? backgoundColor = null, bool rescaleText =true, TextAnchor textAnchor =TextAnchor.MiddleLeft, Color? textColor = null)
        {
            GameObject imageGO = AddChildGOImage(parentGO, backgoundColor ?? Color.gray, anchorPreset, width, height, new Vector2(anchorXOffset, anchorYOffset));
            var textGo = new GameObject("text");
            textGo.transform.SetParent(imageGO.transform, false);
            textGo.AddComponent<RectTransform>();
            Text text = textGo.AddComponent<Text>();
            Font selectedFont = (Font)Resources.GetBuiltinResource(typeof(Font), "Arial.ttf");
            text.font = selectedFont;
            ReformatText(textGo, width - 4f, height, fontSize, 2f, 0, AnchorPresets.centre, textAnchor, resizeTextForBestFit, rescaleText);

            text.horizontalOverflow = horizontalWrapMode;
            //           text.resizeTextForBestFit = resizeTextForBestFit;

            if (resizeTextForBestFit)
            {
                text.resizeTextMaxSize = fontSize;
                text.resizeTextMinSize = minFontSize;
            }
            else text.fontSize = fontSize;

            text.text = textString;

            text.color = textColor ?? Color.white;

            return imageGO;
        }

        public static GameObject CreateBasicTextBox(GameObject parentGO, JSONStorableString jss, float width, float height, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool resizeTextForBestFit = false, int minFontSize = 20, HorizontalWrapMode horizontalWrapMode = HorizontalWrapMode.Wrap, Color? backgoundColor=null, bool rescaleText = true)
        {
            GameObject textImageGO = CreateBasicTextBox( parentGO, jss.val, width, height, fontSize , anchorXOffset, anchorYOffset, anchorPreset, resizeTextForBestFit, minFontSize, horizontalWrapMode, backgoundColor, rescaleText);

            GameObject textGO = textImageGO.transform.Find("text").gameObject;

            jss.RegisterText(textGO.GetComponent<Text>());

            return textImageGO;
        }

        public static UIDynamicTextField CreateTextBox(GameObject parentGO, JSONStorableString jss, float width, float height, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool input = false, InputField.LineType inputFieldType = InputField.LineType.MultiLineNewline, string placeHolderText = "", float hScrollBarHeight = 0f, float vScrollBarWidth = 10f, Action<string> textEntryChangeCallback = null, TextAnchor textAlignment = TextAnchor.MiddleLeft, Action<string> textEntryEndCallback=null, bool rescaleText = true)
        {
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableTextFieldPrefab);

            GameObject textFieldGO = transform.gameObject;
            UIDynamicTextField textFieldUID = textFieldGO.transform.GetComponent<UIDynamicTextField>();
            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(textFieldGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);
            if (input)
            {
                textFieldUID.backgroundColor = Color.white;
                var inputComponent = textFieldUID.gameObject.AddComponent<InputField>();

                inputComponent.textComponent = textFieldUID.UItext;
                inputComponent.lineType = inputFieldType;
                inputComponent.caretWidth = 2;

                inputComponent.textComponent.resizeTextForBestFit = false;
                jss.RegisterInputField(inputComponent);

                if (placeHolderText != "")
                {
                    var placeholderGo = new GameObject("placeHolderText");
                    placeholderGo.transform.SetParent(textFieldUID.gameObject.transform, false);

                    var placeholderRect = placeholderGo.AddComponent<RectTransform>();
                    Font selectedFont = (Font)Resources.GetBuiltinResource(typeof(Font), "Arial.ttf");

                    UnityEngine.UI.Text placeholderUIText = placeholderGo.AddComponent<UnityEngine.UI.Text>();
                    placeholderUIText.raycastTarget = false;
                    placeholderUIText.font = selectedFont;
                    placeholderUIText.color = Color.gray;
                    placeholderUIText.fontStyle = FontStyle.Italic;

                    ReformatText(placeholderGo, width - vScrollBarWidth, height - hScrollBarHeight, fontSize, 0f, 0f, AnchorPresets.middleLeft, TextAnchor.MiddleLeft, rescaleText:rescaleText);

                    if (jss.val == "") placeholderUIText.text = placeHolderText;

                    inputComponent.onValueChanged.AddListener((text) =>
                    {
                        InputTextFieldValueChanged(inputComponent, placeholderUIText, jss, placeHolderText, textEntryChangeCallback);
                    });
                  
                }
                else
                {
                    inputComponent.onValueChanged.AddListener((text) =>
                    {
                        if (textEntryChangeCallback!=null) textEntryChangeCallback(text);
                    });
                }
                inputComponent.onEndEdit.AddListener((text) =>
                {
                    if (textEntryEndCallback != null) textEntryEndCallback(text);
                });

                
            }
            else
            {
                jss.dynamicText = textFieldUID;
            }

            textFieldUID.height = height - hScrollBarHeight;

            GameObject scrollViewGO = textFieldGO.transform.Find("Scroll View").gameObject;
            ReformatScrollView(scrollViewGO, width, vScrollBarWidth, height, hScrollBarHeight);

            GameObject contentGO = textFieldGO.transform.Find("Scroll View/Viewport/Content").gameObject;
            ResizeRecTransform(contentGO, width, height, anchorPreset: GetAnchorPreset(textAlignment));
            AddChildGOImage(contentGO, new Color(0, 0f, 0f, 0f), GetAnchorPreset(textAlignment), width, height, new Vector2(0f, 0f));

            GameObject textContentGO = textFieldGO.transform.Find("Scroll View/Viewport/Content/Text").gameObject;
            ReformatText(textContentGO, width - vScrollBarWidth, height - hScrollBarHeight, fontSize, 2f, 0f, anchorPreset: GetAnchorPreset(textAlignment), textAlginment: textAlignment, rescaleText: rescaleText);

            if (input && inputFieldType != InputField.LineType.SingleLine)
            {
                var csf = jss.inputField.textComponent.gameObject.AddComponent<ContentSizeFitter>();
                csf.verticalFit = ContentSizeFitter.FitMode.PreferredSize;
                jss.inputField.textComponent.horizontalOverflow = HorizontalWrapMode.Wrap;
                jss.inputField.textComponent.verticalOverflow = VerticalWrapMode.Truncate;
            }

            return textFieldUID;
        }



        private static int GetAnchorPreset(TextAnchor textAnchor)
        {
            if (textAnchor == TextAnchor.UpperLeft) return AnchorPresets.topLeft;
            if (textAnchor == TextAnchor.UpperCenter) return AnchorPresets.topMiddle;
            if (textAnchor == TextAnchor.UpperRight) return AnchorPresets.topRight;

            if (textAnchor == TextAnchor.MiddleLeft) return AnchorPresets.middleLeft;
            if (textAnchor == TextAnchor.MiddleCenter) return AnchorPresets.centre;
            if (textAnchor == TextAnchor.UpperRight) return AnchorPresets.middleRight;

            if (textAnchor == TextAnchor.LowerLeft) return AnchorPresets.bottomLeft;
            if (textAnchor == TextAnchor.LowerCenter) return AnchorPresets.bottomMiddle;
            if (textAnchor == TextAnchor.LowerRight) return AnchorPresets.bottomRight;

            return -1;
        }
        private static void InputTextFieldValueChanged(InputField inputField, Text placeholderUIText, JSONStorableString jss, string placeHolderText, Action<string> textEntryChangeCallback)
        {
            if (inputField.text == "") placeholderUIText.text = placeHolderText;
            else placeholderUIText.text = "";

            if (textEntryChangeCallback != null) textEntryChangeCallback(inputField.text);
        }

        public static UIDynamicButton CreateUIButton(GameObject parentGO, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool clearSprite = false, bool rescaleText = true)
        {
            Transform buttonTransform = UnityEngine.Object.Instantiate(UIASceneButtonsSetup.mvrScript.manager.configurableButtonPrefab.transform, parentGO.transform, false);

            GameObject buttonGO = buttonTransform.gameObject;
            buttonTransform.SetParent(parentGO.transform, false);

            UIDynamicButton buttonUID = buttonGO.transform.GetComponent<UIDynamicButton>();
            Transform textTransform = buttonUID.button.transform.Find("Text");
            ReformatText(textTransform.GetComponent<RectTransform>().gameObject, width, height, fontSize, 0, 0, AnchorPresets.centre,rescaleText:rescaleText);

            buttonUID.label = label;
            buttonUID.buttonText.fontSize = fontSize;

            if (clearSprite)
            {
                Image buttonImage = buttonGO.transform.GetComponent<Image>();
                buttonImage.sprite = null;
            }

            ResizeRecTransform(buttonGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            return (buttonUID);

        }

        public static UIDynamicSlider CreateSlider(GameObject parentGO, JSONStorableFloat jsf, float width, float height, string label = null, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, float quickButtonHeight =0f, float labelHeight =0f, float rangeWidth=0f, bool hideRange=false, bool hideRangeLabel = false, bool rescaleText =true)
        {
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableSliderPrefab);

            GameObject sliderGO = transform.gameObject;

            
            UIDynamicSlider sliderUID = sliderGO.transform.GetComponent<UIDynamicSlider>();
            if (label != null) sliderUID.label = label;
            jsf.RegisterSlider(sliderUID.slider);

            transform.SetParent(parentGO.transform, false);
            ResizeRecTransform(sliderGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            float sliderHeight = height - quickButtonHeight - labelHeight;
            float sliderWidth = width - rangeWidth;
            if (labelHeight == 0f) sliderWidth = width - (rangeWidth * 2);
            if (hideRange) sliderWidth = sliderWidth + rangeWidth;

            GameObject subSliderGO = sliderGO.transform.Find("Slider").gameObject;
            ResizeRecTransform(subSliderGO, sliderWidth, sliderHeight, 0f, -labelHeight, AnchorPresets.topLeft);

            float qbWidth = width / 9f;
            float qbGroupWidth = qbWidth * 4f;
            GameObject quickButtonsGroupGO = sliderGO.transform.Find("QuickButtonsGroup").gameObject;
            ResizeRecTransform(quickButtonsGroupGO, width, quickButtonHeight, 0f, 0f, AnchorPresets.bottomMiddle);
                       
            GameObject quickButtonsLeftGO = sliderGO.transform.Find("QuickButtonsGroup/QuickButtonsLeft").gameObject;
            ResizeRecTransform(quickButtonsLeftGO, qbGroupWidth, quickButtonHeight, 0f, 0f, AnchorPresets.bottomLeft);
            
            ResizeSliderQuickButton("M4Button", quickButtonsLeftGO, qbWidth, quickButtonHeight, qbWidth*0);
            ResizeSliderQuickButton("M3Button", quickButtonsLeftGO, qbWidth, quickButtonHeight, qbWidth*1);
            ResizeSliderQuickButton("M2Button", quickButtonsLeftGO, qbWidth, quickButtonHeight, qbWidth*2);
            ResizeSliderQuickButton("M1Button", quickButtonsLeftGO, qbWidth, quickButtonHeight, qbWidth*3);

            GameObject quickButtonsRightGO = sliderGO.transform.Find("QuickButtonsGroup/QuickButtonsRight").gameObject;
            ResizeRecTransform(quickButtonsRightGO, qbGroupWidth, quickButtonHeight, 0f, 0f, AnchorPresets.bottomRight);

            ResizeSliderQuickButton("P1Button", quickButtonsRightGO, qbWidth, quickButtonHeight, qbWidth * 0);
            ResizeSliderQuickButton("P2Button", quickButtonsRightGO, qbWidth, quickButtonHeight, qbWidth * 1);
            ResizeSliderQuickButton("P3Button", quickButtonsRightGO, qbWidth, quickButtonHeight, qbWidth * 2);
            ResizeSliderQuickButton("P4Button", quickButtonsRightGO, qbWidth, quickButtonHeight, qbWidth * 3);

            GameObject quickButtonsDefaultGO = sliderGO.transform.Find("DefaultValueButton").gameObject;
            ReformatUnityButton(quickButtonsDefaultGO, qbWidth, quickButtonHeight, 14, 0, 0, AnchorPresets.bottomMiddle, true);

            GameObject textGO = sliderGO.transform.Find("Text").gameObject;
            ReformatText(textGO, width * 0.7f, labelHeight, fontSize, 0, 0, AnchorPresets.topLeft, TextAnchor.MiddleLeft, false, rescaleText: rescaleText);
            if (labelHeight == 0) textGO.SetActive(false);

            GameObject inputGO = sliderGO.transform.Find("ValueInputField").gameObject;
            if (labelHeight > 0f)
            {
                ResizeRecTransform(inputGO, width * 0.3f, labelHeight, 0f, 0f, AnchorPresets.topRight);
                GameObject inputPlaceHolderGO = inputGO.transform.Find("Placeholder").gameObject;
                ResizeRecTransform(inputPlaceHolderGO, width * 0.3f, labelHeight, 0f, 0f, AnchorPresets.topRight);
                GameObject inputTextGO = inputGO.transform.Find("Text").gameObject;
                ReformatText(inputTextGO, width * 0.3f, labelHeight, fontSize, 0, 0, AnchorPresets.topRight, TextAnchor.MiddleRight, false,rescaleText);
            }
            else
            {
                float inputOffset = rangeWidth;
                if (hideRange) inputOffset = 0f;
                ResizeRecTransform(inputGO, rangeWidth, sliderHeight, -inputOffset, 0f, AnchorPresets.topRight);
                GameObject inputPlaceHolderGO = inputGO.transform.Find("Placeholder").gameObject;
                ResizeRecTransform(inputPlaceHolderGO, rangeWidth, sliderHeight, 0f, 0f, AnchorPresets.topRight);
                GameObject inputTextGO = inputGO.transform.Find("Text").gameObject;
                ReformatText(inputTextGO, rangeWidth, sliderHeight, fontSize, -0f, 0, AnchorPresets.topRight, TextAnchor.MiddleCenter, false,rescaleText);
            }

            float rangeButtonHeight = sliderHeight / 2f;
            if (hideRangeLabel) rangeButtonHeight = sliderHeight;

            GameObject rangeGO = sliderGO.transform.Find("RangePanel").gameObject;
            ResizeRecTransform(rangeGO, rangeWidth*.95f, sliderHeight * .95f, 0f, -labelHeight, AnchorPresets.topRight);
            GameObject rangeTextGO = rangeGO.transform.Find("RangeText").gameObject;
            ReformatText(rangeTextGO, rangeWidth * .95f, sliderHeight / 2f, fontSize, 0, 0, AnchorPresets.topRight, TextAnchor.MiddleCenter, true, rescaleText);
            if (hideRangeLabel) rangeTextGO.SetActive(false);
            GameObject x10RangeButtonGO = rangeGO.transform.Find("10XRangeButton").gameObject;
            ReformatUnityButton(x10RangeButtonGO, rangeWidth/2f, rangeButtonHeight*0.8f, 20, 0, 1f, AnchorPresets.bottomRight,true,resizeTextForBestFit:false);
            GameObject x01RangeButtonGO = rangeGO.transform.Find("P1XRangeButton").gameObject;
            ReformatUnityButton(x01RangeButtonGO, rangeWidth / 2f, rangeButtonHeight * 0.8f, 20, 0, 1f, AnchorPresets.bottomLeft, true, resizeTextForBestFit: false);
            if (hideRange) rangeGO.SetActive(false);

            return sliderUID;
        }

        private static void ResizeSliderQuickButton(string transformTag, GameObject parentGO, float width, float height, float xOffset)
        {
            GameObject quickButtonsLeftMxGO = parentGO.transform.Find(transformTag).gameObject;
            ReformatUnityButton(quickButtonsLeftMxGO, width, height, 14, xOffset, 0, AnchorPresets.bottomLeft,true,resizeTextForBestFit:true);
        }

        public static UIDynamicToggle CreateToggle(GameObject parentGO, JSONStorableBool jsb, float width, float height, string label = null, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, float checkBoxSize = 20f, bool rescaleText=true)
        {
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableTogglePrefab);

            GameObject toggleGO = transform.gameObject;
            UIDynamicToggle toggleUID = toggleGO.transform.GetComponent<UIDynamicToggle>();
            if (label!=null) toggleUID.label = label;
            jsb.RegisterToggle(toggleUID.toggle);

            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(toggleGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject labelTextGO = toggleGO.transform.Find("Label").gameObject;
            ReformatText(labelTextGO, width - checkBoxSize, height, fontSize, 0f, 0f, AnchorPresets.middleRight, textAlginment: TextAnchor.MiddleLeft,rescaleText:rescaleText);

            GameObject checkBoxGO = toggleGO.transform.Find("Background").gameObject;
            ResizeRecTransform(checkBoxGO, checkBoxSize, checkBoxSize, 0f, 0f, AnchorPresets.middleLeft);

            GameObject checkMarkGO = toggleGO.transform.Find("Background/Checkmark").gameObject;
            ResizeRecTransform(checkMarkGO, checkBoxSize, checkBoxSize, 0f, 0f, AnchorPresets.centre);

            return toggleUID;
        }


        public static UIDynamicPopup CreatePopup(GameObject parentGO, JSONStorableStringChooser jssc, float width, float height, string label = null, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool popupAbove = false, float labelProportion = 0.4f, float popupRowHeight = 15f, bool rescaleText=true)
        {
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurablePopupPrefab);

            GameObject popupGO = transform.gameObject;
            UIDynamicPopup uiPopup = popupGO.transform.GetComponent<UIDynamicPopup>();
           
            uiPopup.popup.selectColor = Color.cyan;

            jssc.RegisterPopup(uiPopup.popup);
            if (label != null) uiPopup.popup.label = label;

            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(popupGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject dropDownButtonGO = transform.Find("Button").gameObject;
            ReformatUnityButton(dropDownButtonGO, width * (1 - labelProportion), height * .9f, fontSize, 0f, 0f, AnchorPresets.middleRight, rescaleText:rescaleText);

            GameObject textGO = transform.Find("Text").gameObject;
            ReformatText(textGO, width * (labelProportion), height * .9f, fontSize, 0f, 0f, AnchorPresets.middleLeft,rescaleText:rescaleText);

            GameObject popupPanelGO = transform.Find("PopupPanel").gameObject;
            
            if (UIASceneButtonsSetup.isVREnabled) popupPanelGO.transform.Translate(new Vector3(0, 0, 0.06f));

            if (popupAbove) ReformatPopupPanel(popupPanelGO, width * (1 - labelProportion), 300f, fontSize, 0f, height, anchorPreset: AnchorPresets.bottomRight, popupRowHeight: popupRowHeight, rescaleText: rescaleText);
            else ReformatPopupPanel(popupPanelGO, width * (1 - labelProportion), 300f, fontSize, 0f, -height, anchorPreset: AnchorPresets.topRight, popupRowHeight: popupRowHeight, rescaleText: rescaleText);

            return uiPopup;
        }
        public static UIDynamicPopup CreateScrollablePopup(GameObject parentGO, JSONStorableStringChooser jssc, float width, float height, string label = null, int labelFontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, float popupHeight = 300f, bool popupAbove = false, bool disableSlider = false, int anchorPreset = AnchorPresets.none, float labelProportion = 0.4f, float popupRowHeight = 15f,int popupFontSize = 20, bool rescaleText =true, float scrollBarWidth = 10f)
        {
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableScrollablePopupPrefab);

            GameObject popupGO = transform.gameObject;
            UIDynamicPopup uiPopup = popupGO.transform.GetComponent<UIDynamicPopup>();
            
            jssc.RegisterPopup(uiPopup.popup);
            if (label != null) uiPopup.popup.label = label;

            float sliderHeight = 10f;
            if (disableSlider) sliderHeight = 0f;

            uiPopup.popup.selectColor = Color.cyan;
            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(popupGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject dropDownButtonGO = transform.Find("Button").gameObject;
            ReformatUnityButton(dropDownButtonGO, width * (1 - labelProportion), (height * .9f) - sliderHeight, popupFontSize, -2f, 0f, AnchorPresets.middleRight, true,rescaleText:rescaleText);

            GameObject textGO = transform.Find("Text").gameObject;
            ReformatText(textGO, width * (labelProportion), height * .9f, labelFontSize, 0f, 0f, AnchorPresets.middleLeft, rescaleText: rescaleText);

            GameObject popupPanelGO = transform.Find("PopupPanel").gameObject;
            GameObject scrollViewGO = popupPanelGO.transform.Find("Scroll View").gameObject;

            ScrollRect scrollRect = scrollViewGO.GetComponent<ScrollRect>();
            scrollRect.verticalScrollbarVisibility = ScrollRect.ScrollbarVisibility.AutoHideAndExpandViewport;

            if (popupAbove) ReformatPopupPanel(popupPanelGO, width * (1 - labelProportion), popupHeight, popupFontSize, 0f, height, anchorPreset: AnchorPresets.bottomRight, popupRowHeight: popupRowHeight, rescaleText: rescaleText, scrollBarWidth: scrollBarWidth);
            else ReformatPopupPanel(popupPanelGO, width * (1 - labelProportion), popupHeight, popupFontSize, 0f, -height, anchorPreset: AnchorPresets.topRight, popupRowHeight: popupRowHeight, rescaleText: rescaleText, scrollBarWidth: scrollBarWidth);

            GameObject sliderGO = transform.Find("Slider").gameObject;
            ReformatScrollPopupSlider(sliderGO, width * (1 - labelProportion), sliderHeight, 0f, 0f, anchorPreset: AnchorPresets.topRight, clearSprite: false);

            return uiPopup;
        }


        public static void RecursiveTrans(Transform transform, string spaces="")
        {

            foreach (Transform childTrans in transform)
            {
                RectTransform rt = transform.gameObject.GetComponent<RectTransform>();


                SuperController.LogMessage(spaces + childTrans.name);
                string newSpaces = spaces + "    ";
                RecursiveTrans(childTrans, newSpaces);
            }
        }

        public static UIDynamicColorPicker CreateColorPicker(GameObject parentGO,JSONStorableColor jsc, float width, float height, string label = "",float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none){
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableColorPickerPrefab);

            GameObject colorPickerGO = transform.gameObject;
            UIDynamicColorPicker uiColorPicker = colorPickerGO.transform.GetComponent<UIDynamicColorPicker>();

            jsc.RegisterColorPicker(uiColorPicker.colorPicker);

            if (label!=null) uiColorPicker.label = label;

            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(colorPickerGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            return uiColorPicker;
        }

        public static UIDynamicPopup CreateFilterablePopup(GameObject parentGO, JSONStorableStringChooser jssc, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, float popupHeight = 300f, int anchorPreset = AnchorPresets.none, bool rescaleText = true)
        {
            float labelProportion = 0.3f;
            Transform transform = GameObject.Instantiate<Transform>(UIASceneButtonsSetup.mvrScript.manager.configurableFilterablePopupPrefab);

            GameObject popupGO = transform.gameObject;

            UIDynamicPopup uiPopup = popupGO.transform.GetComponent<UIDynamicPopup>();
            uiPopup.label = label;
            uiPopup.popup.selectColor = Color.cyan;
            jssc.RegisterPopup(uiPopup.popup);

            transform.SetParent(parentGO.transform, false);

            ResizeRecTransform(popupGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject dropDownButtonGO = transform.Find("Button").gameObject;
            ReformatUnityButton(dropDownButtonGO, width * (1 - labelProportion), height * 0.6f, fontSize, 0f, 0f, AnchorPresets.bottomRight, false);

            GameObject labelTextGO = transform.Find("Text").gameObject;
            ReformatText(labelTextGO, width * labelProportion, height, fontSize, 0f, 0f, AnchorPresets.middleLeft, rescaleText: rescaleText);

            GameObject inputFieldGO = transform.Find("InputField").gameObject;
            ReformatFilterPopupInputField(inputFieldGO, width * (1 - labelProportion), height * 0.4f, fontSize, 0f, 0f, AnchorPresets.topRight);

            GameObject popupPanelGO = transform.Find("PopupPanel").gameObject;
            ReformatPopupPanel(popupPanelGO, width * (1 - labelProportion), popupHeight, fontSize, 0f, -height, anchorPreset: AnchorPresets.topRight, rescaleText: rescaleText);
            GameObject scrollViewGO = popupPanelGO.transform.Find("Scroll View").gameObject;

            ScrollRect scrollRect = scrollViewGO.GetComponent<ScrollRect>();
            scrollRect.verticalScrollbarVisibility = ScrollRect.ScrollbarVisibility.AutoHideAndExpandViewport;
            return uiPopup;
        }
        private static void ReformatPopupPanel(GameObject popupPanelGO, float width, float height, int fontSize = 12, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, float popupRowHeight = 15f, bool rescaleText = true, float scrollBarWidth = 10f)
        {
            ResizeRecTransform(popupPanelGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject buttonTemplateGO = popupPanelGO.transform.Find("PopupValueButtonTemplate").gameObject;
            ReformatUnityButton(buttonTemplateGO, width - scrollBarWidth, popupRowHeight, fontSize, clearSprite: true, anchorPreset: AnchorPresets.none, rescaleText: rescaleText);

            if (popupPanelGO.transform.Find("Scroll View") != null)
            {
                ReformatScrollView(popupPanelGO.transform.Find("Scroll View").gameObject, width, scrollBarWidth, height);
            }
        }

        private static void ReformatScrollView(GameObject scrollViewGO, float width, float vScrollBarWidth, float height = 25, float hScrollBarHeight = 0)
        {
            GameObject viewPortGO = scrollViewGO.transform.Find("Viewport").gameObject;
            ResizeRecTransform(viewPortGO, width-vScrollBarWidth, height- hScrollBarHeight);

            RectTransform viewPortRT = viewPortGO.transform.GetComponent<RectTransform>();
            viewPortRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height - hScrollBarHeight / 2);
            viewPortRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width - vScrollBarWidth / 2);

            if (scrollViewGO.transform.Find("Scrollbar Vertical") != null)
            {
                GameObject scrollBarVerticalGO = scrollViewGO.transform.Find("Scrollbar Vertical").gameObject;
                Scrollbar sb = scrollBarVerticalGO.transform.GetComponent<Scrollbar>();
                sb.enabled = vScrollBarWidth > 0f;
                
                RectTransform scrollBarRT = scrollBarVerticalGO.transform.GetComponent<RectTransform>();
                scrollBarRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, vScrollBarWidth);
                scrollBarRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            }
            if (scrollViewGO.transform.Find("Scrollbar Horizontal") != null)
            {
                GameObject scrollBarVerticalGO = scrollViewGO.transform.Find("Scrollbar Horizontal").gameObject;
                Scrollbar sb = scrollBarVerticalGO.transform.GetComponent<Scrollbar>();
                sb.enabled = hScrollBarHeight > 0f;
                RectTransform scrollBarRT = scrollBarVerticalGO.transform.GetComponent<RectTransform>();
                scrollBarRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, hScrollBarHeight);
                scrollBarRT.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width  );

            }


        }

        private static void ReformatScrollPopupSlider(GameObject sliderGO, float width, float height, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool clearSprite = true)
        {
            ResizeRecTransform(sliderGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject handleGO = sliderGO.transform.Find("Handle Slide Area/Handle").gameObject;
            Image handleImage = handleGO.transform.GetComponent<Image>();
            if (clearSprite) handleImage.sprite = null;
        }

        private static void ReformatImage(GameObject imageGO, float width, float height, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool clearSprite = true)
        {
            ResizeRecTransform(imageGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);
            Image popupImage = imageGO.transform.GetComponent<Image>();
            if (clearSprite) popupImage.sprite = null;
            popupImage.color = new Color(0.9f, 0.9f, 0.9f);
        }
        private static void ReformatFilterPopupInputField(GameObject inputFieldGO, float width, float height, int fontSize = 12, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool rescaleText = true)
        {
            Image popupImage = inputFieldGO.transform.GetComponent<Image>();
            popupImage.sprite = null;

            ResizeRecTransform(inputFieldGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            GameObject buttonGO = inputFieldGO.transform.Find("Button").gameObject;
            ReformatUnityButton(buttonGO, 15f, height, fontSize - 4, -40f, clearSprite: false, textAlignment: TextAnchor.MiddleLeft);

            GameObject countTextGO = inputFieldGO.transform.Find("CountText").gameObject;
            ReformatText(countTextGO, 40f, height, fontSize - 4, anchorPreset: AnchorPresets.middleRight, rescaleText: rescaleText);

            GameObject countTextBGGO = inputFieldGO.transform.Find("CountTextBG").gameObject;
            ReformatImage(countTextBGGO, 40f, height, anchorPreset: AnchorPresets.middleRight, clearSprite: true);
            GameObject maxCountBGGO = inputFieldGO.transform.Find("MaxIndicator").gameObject;
            ReformatImage(maxCountBGGO, 40f, height, anchorPreset: AnchorPresets.middleRight, clearSprite: true);

            GameObject inputTextGO = inputFieldGO.transform.Find("Text").gameObject;
            ReformatText(inputTextGO, width - 55f, height, fontSize, 0f, 0f, AnchorPresets.middleLeft, textAlginment: TextAnchor.MiddleLeft, rescaleText: rescaleText);

            GameObject placeholderTextGO = inputFieldGO.transform.Find("Placeholder").gameObject;
            ReformatText(placeholderTextGO, width - 55f, height, fontSize, 0f, 0f, AnchorPresets.middleLeft, textAlginment: TextAnchor.MiddleLeft, rescaleText: rescaleText);
        }

        public static void ReformatText(GameObject textGO, float width, float height, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, TextAnchor textAlginment = TextAnchor.MiddleCenter, bool resizeTextForBestFit = false, bool rescaleText = true)
        {

            RectTransform textRT = textGO.transform.GetComponent<RectTransform>();

            if (rescaleText)
            {
                textRT.localScale = new Vector3(0.5f, 0.5f, 0.5f);
                width = width * 2;
                height = height * 2;
            }

            ResizeRecTransform(textGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            Text text = textGO.transform.GetComponent<Text>();
            text.alignment = textAlginment;
            text.fontSize = fontSize;
            text.resizeTextForBestFit = resizeTextForBestFit;
        }
        private static void ReformatUnityButton(GameObject buttonGO, float width, float height, int fontSize = 12, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool clearSprite = true, TextAnchor textAlignment = TextAnchor.MiddleCenter, bool resizeTextForBestFit = false, bool rescaleText=true)
        {
            ResizeRecTransform(buttonGO, width, height, anchorXOffset, anchorYOffset, anchorPreset);

            if (clearSprite)
            {
                Image buttonImage = buttonGO.transform.GetComponent<Image>();
                buttonImage.sprite = null;
            }

            Transform textTransform = buttonGO.transform.Find("Text");
            ReformatText(textTransform.gameObject, width - 4f, height, fontSize, 2f, 0f, AnchorPresets.centre, textAlignment,resizeTextForBestFit, rescaleText);
        }

        public static void ResizeRecTransform(GameObject go, float width, float height, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            ResizeRecTransform(rt, width, height, anchorXOffset, anchorYOffset, anchorPreset);

        }

        public static void ResizeRecTransform(RectTransform rt, float width, float height, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none)
        {
            rt.sizeDelta = Vector2.zero;
            if (anchorPreset != AnchorPresets.none)
            {
                
                rt.anchorMin = AnchorPresets.GetAnchorMin(anchorPreset);
                rt.anchorMax = AnchorPresets.GetAnchorMax(anchorPreset);
                rt.pivot = AnchorPresets.GetPivot(anchorPreset);
            }
            rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
            rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
            rt.anchoredPosition = new Vector2(anchorXOffset, anchorYOffset);
        }

        public static void ResizeWidth(GameObject go, float width)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Horizontal, width);
        }

        public static void ResizeHeight(GameObject go, float height)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical, height);
        }

        public static void ResetBottomOffset(GameObject go, float offset)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.offsetMin = new Vector2(rt.offsetMin.x, offset);
        }

        public static void ResetTopOffset(GameObject go, float offset)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.offsetMax = new Vector2(rt.offsetMax.x, offset);
        }

        public static void ResetAnchorPosition(GameObject go, float xPosition, float yPosition)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.anchoredPosition = new Vector2(xPosition, yPosition);
        }
        public static void ResetAnchorType(GameObject go, int anchorPreset)
        {
            RectTransform rt = go.GetComponent<RectTransform>();
            rt.anchorMin = AnchorPresets.GetAnchorMin(anchorPreset);
            rt.anchorMax = AnchorPresets.GetAnchorMax(anchorPreset);
            rt.pivot = AnchorPresets.GetPivot(anchorPreset);
        }

    }


    public static class AnchorPresets
    {
        public const int none = 0;
        public const int topLeft = 1;
        public const int topMiddle = 2;
        public const int topRight = 3;
        public const int middleLeft = 4;
        public const int centre = 5;
        public const int middleRight = 6;
        public const int bottomLeft = 7;
        public const int bottomMiddle = 8;
        public const int bottomRight = 9;

        public const int vStretchMiddle = 11;
        public const int hStretchMiddle = 12;
        public const int vStretchRight = 13;
        public const int vStretchLeft = 14;
        public const int hStretchTop = 15;
        public const int hStretchBottom = 16;
        public const int vhStretch = 17;

        public static Vector2 GetAnchorMin(int anchorPreset)
        {
            Vector2 anchorMin = new Vector2();

            if (anchorPreset == topLeft) anchorMin = new Vector2(0f, 1f);
            if (anchorPreset == topMiddle) anchorMin = new Vector2(0.5f, 1f);
            if (anchorPreset == topRight) anchorMin = new Vector2(1f, 1f);
            if (anchorPreset == middleLeft) anchorMin = new Vector2(0f, 0.5f);
            if (anchorPreset == centre) anchorMin = new Vector2(0.5f, 0.5f);
            if (anchorPreset == middleRight) anchorMin = new Vector2(1f, 0.5f);
            if (anchorPreset == bottomLeft) anchorMin = new Vector2(0f, 0f);
            if (anchorPreset == bottomMiddle) anchorMin = new Vector2(0.5f, 0f);
            if (anchorPreset == bottomRight) anchorMin = new Vector2(1f, 0f);

            if (anchorPreset == vStretchMiddle) anchorMin = new Vector2(0.5f, 0f);
            if (anchorPreset == vStretchRight) anchorMin = new Vector2(1f, 0f);
            if (anchorPreset == vStretchLeft) anchorMin = new Vector2(0f, 0f);
            if (anchorPreset == hStretchMiddle) anchorMin = new Vector2(0f, 0.5f);
            if (anchorPreset == hStretchTop) anchorMin = new Vector2(0f, 1f);
            if (anchorPreset == hStretchBottom) anchorMin = new Vector2(0f, 0f);
            if (anchorPreset == vhStretch) anchorMin = new Vector2(0f, 0f);

            return anchorMin;
        }

        public static Vector2 GetAnchorMax(int anchorPreset)
        {
            Vector2 anchorMax = new Vector2();

            if (anchorPreset == topLeft) anchorMax = new Vector2(0f, 1f);
            if (anchorPreset == topMiddle) anchorMax = new Vector2(0.5f, 1f);
            if (anchorPreset == topRight) anchorMax = new Vector2(1f, 1f);
            if (anchorPreset == middleLeft) anchorMax = new Vector2(0f, 0.5f);
            if (anchorPreset == centre) anchorMax = new Vector2(0.5f, 0.5f);
            if (anchorPreset == middleRight) anchorMax = new Vector2(1f, 0.5f);
            if (anchorPreset == bottomLeft) anchorMax = new Vector2(0f, 0f);
            if (anchorPreset == bottomMiddle) anchorMax = new Vector2(0.5f, 0f);
            if (anchorPreset == bottomRight) anchorMax = new Vector2(1f, 0f);

            if (anchorPreset == vStretchMiddle) anchorMax = new Vector2(0.5f, 1f);
            if (anchorPreset == vStretchRight) anchorMax = new Vector2(1f, 1f);
            if (anchorPreset == vStretchLeft) anchorMax = new Vector2(0f, 1f);
            if (anchorPreset == hStretchMiddle) anchorMax = new Vector2(1f, 0.5f);
            if (anchorPreset == hStretchTop) anchorMax = new Vector2(1f, 1f);
            if (anchorPreset == hStretchBottom) anchorMax = new Vector2(1f, 0f);
            if (anchorPreset == vhStretch) anchorMax = new Vector2(1f, 1f);

            return anchorMax;
        }

        public static Vector2 GetPivot(int anchorPreset)
        {
            Vector2 pivot = new Vector2();

            if (anchorPreset == topLeft) pivot = new Vector2(0f, 1f);
            if (anchorPreset == topMiddle) pivot = new Vector2(0.5f, 1f);
            if (anchorPreset == topRight) pivot = new Vector2(1f, 1f);
            if (anchorPreset == middleLeft) pivot = new Vector2(0f, 0.5f);
            if (anchorPreset == centre) pivot = new Vector2(0.5f, 0.5f);
            if (anchorPreset == middleRight) pivot = new Vector2(1f, 0.5f);
            if (anchorPreset == bottomLeft) pivot = new Vector2(0f, 0f);
            if (anchorPreset == bottomMiddle) pivot = new Vector2(0.5f, 0f);
            if (anchorPreset == bottomRight) pivot = new Vector2(1f, 0f);

            if (anchorPreset == vStretchMiddle) pivot = new Vector2(0.5f, 1f);
            if (anchorPreset == hStretchMiddle) pivot = new Vector2(0f, 0.5f);
            if (anchorPreset == vStretchRight) pivot = new Vector2(1f, 1f);
            if (anchorPreset == vStretchLeft) pivot = new Vector2(0f, 1f);
            if (anchorPreset == hStretchTop) pivot = new Vector2(0f, 1f);
            if (anchorPreset == hStretchBottom) pivot = new Vector2(0f, 0f);
            if (anchorPreset == vhStretch) pivot = new Vector2(0f, 1f);

            return pivot;
        }

    }


}
