﻿using UnityEngine;
using UnityEngine.UI;

namespace JayJayWon
{
    public class SceneButtonConfigsUI : BaseUITab
    {
        private CustomUIElement sceneButtonConfigsUIElement;
        private CustomUIElement helpTextCustomUI;
        public JSONStorableStringChooser aciveButtonConfigJSSC {get {return UIASceneButtonsSetup.activeButtonConfigJSSC;}}

        private int uiMode = UIMode.configManager;
        public UIASceneButtonConfig modifyingConfig{get;private set;}

        public JSONStorableStringChooser modifyingConfigJSSC ;
        public UIASceneButton modifyingButton {get;private set;}
        public JSONStorableStringChooser modifyingButtonJSSC ;
        private JSONStorableString tempConfigNameJSS ;
        private JSONStorableString tempButtonNameJSS ;

        public SceneButtonConfigsUI()
        {
            uiGroupCount = 2;

            sceneButtonConfigsUIElement = new CustomUIElement(975);
            helpTextCustomUI = new CustomUIElement(200);
            defaultHelpText = "";
            helpTextJSS.val = defaultHelpText;

            modifyingConfigJSSC = new JSONStorableStringChooser("modifyingConfig",UIASceneButtonConfig.configNames,"","",ModifyingConfigChangeCallback);
            modifyingButtonJSSC = new JSONStorableStringChooser("modifyingButton",null,"","",ModifyingButtonChangeCallback);

            tempConfigNameJSS = new JSONStorableString("tempConfigString","");
            tempButtonNameJSS = new JSONStorableString("tempButtonString","");
        }


        public void SetConfigMgrUIMode()
        {
            uiMode = UIMode.configManager;
            modifyingConfigJSSC.valNoCallback = "";
            modifyingButtonJSSC.valNoCallback = "";
            tempButtonNameJSS.valNoCallback ="";
            RefreshGroupUI(1);
        }

        public void RefreshModifyingConfigNameChoices(){
            modifyingConfigJSSC.choices = UIASceneButtonConfig.configNames;
        }
        public void RefreshButtonNameChoices(){
            modifyingButtonJSSC.choices = modifyingConfig.buttonNames;
        }

        private void ModifyingConfigChangeCallback(string modifyingConfigName){
            modifyingConfig = UIASceneButtonConfig.GetConfig(modifyingConfigName);
            RefreshGroupUI(1);

        }

        private void ModifyingButtonChangeCallback(string modifyingButtonName){
            modifyingButton = modifyingConfig.GetButton(modifyingButtonName);
            RefreshGroupUI(1);

        }

        public void SetModifyConfigUIMode(UIASceneButtonConfig modifyingConfig)
        {
                uiMode = UIMode.configSetup;
                modifyingConfigJSSC.val = modifyingConfig.configName;
                
        }
        public void SetModifyButtonUIMode(UIASceneButton modifyingButton)
        {
            uiMode = UIMode.buttonSetup;
            modifyingButtonJSSC.val = modifyingButton.buttonName;
        }

        protected override void CreateGroupUI(int groupID)
        {
            if (groupID == 1)
            {
                sceneButtonConfigsUIElement.InitUI(CustomRowWidth.full);
                RegisterCustomUIElement(groupID, sceneButtonConfigsUIElement);
                if (uiMode==UIMode.configManager) CreateConfigMgrUI();
                if (uiMode==UIMode.configSetup || uiMode==UIMode.configRename) CreateConfigSetupUI();
                if (uiMode==UIMode.buttonSetup || uiMode==UIMode.buttonRename) CreateButtonSetupUI();


            }
            if (groupID == 2)
            {
                helpTextCustomUI.InitUI(CustomRowWidth.full);

                var helpTextBox = helpTextCustomUI.AddTextBox(helpTextJSS, 1050f, 200f, 0f, 0f, textAlignment:TextAnchor.UpperLeft, vScrollBarWidth:20f);
                helpTextBox.textColor = Color.white;
                helpTextBox.backgroundColor = new Color(0.4f, 0.5f, 0.2f, 0.4f);

                RegisterCustomUIElement(groupID, helpTextCustomUI);
            }
        }
        
        private void CreateConfigMgrUI(){
            defaultHelpText = "UIASceneButtonSetup is a scene based plugin that integrates with the UIAssist Session plugin by JayJayWon. It allows Scene creators to embed scene specific buttons within the UIAssist interface and setup VAM trigger actions to occur when each button is activated by the UIAssist user. Multiple different configurations of buttons can be setup and then dynamically switched between. For example, one setup of buttons can be presented to the UIAssist user on scene load and then different sets of buttons presented as events occur within the scene.\n\nThis Config Manager screen allows for Configs to be created, deleted, re-ordered and modified. The Active Config can be set here (or externally with VAM Triggers), determining which of the Configs will be visible in UIAssist. The first Config in the list will always be the active config on scene load. If the Active Config has no buttons configured then it will not be visible in UIAssist.\n\nThese button configs will be persisted in a Scene save file and reloaded automatically when the scene is loaded.";

            if (!UIASceneButtonConfig.configNames.Contains (aciveButtonConfigJSSC.val)) aciveButtonConfigJSSC.val = UIASceneButtonConfig.configNames[0];  

            var configMrgTxtBox = sceneButtonConfigsUIElement.AddTextBox("Config Manager",250f,70,0,0);         

            if (UIASceneButtonsSetup.currentlyRegisteredConfig!=null) {
                var activeConfigPopup = sceneButtonConfigsUIElement.AddPopup(aciveButtonConfigJSSC,RefreshActiveConfigNameChoices,900,70,0,-100,400f,50f);
                ApplyHelp(activeConfigPopup,"The selected Config will be the 'Active' Config of Buttons that UIAssist will display as options to the UIAssist user. If the Active Config contains no configured buttons, then no options will be available in UIAssist.");
            }
            else{
                var warningTextBox = sceneButtonConfigsUIElement.AddTextBox("WARNING: No compatible version of UIAssist is registering with this plugin",900,70,0,-100);
            }
            var addButton = sceneButtonConfigsUIElement.AddButton(70f,70f,955,-100,"+",AddNewConfig);
            addButton.buttonText.fontSize = 48;
            addButton.buttonColor = new Color(0,0.7f,0);
            addButton.textColor = Color.white;
            ApplyHelp(addButton,"Create a new Config of Buttons. Each Config can define a different set of Buttons that are preseted to the UIAssist user.");

            var sceneButtonConfigsContentGO = sceneButtonConfigsUIElement.CreateVScrollableContent(AnchorPresets.topLeft,1050,800,new Vector2(0,-175),10,2);

            foreach (var sceneButtonConfig in UIASceneButtonConfig.buttonConfigs){
                var sceneButtonsConfigRow = new SceneButtonsConfigRowItem(sceneButtonConfigsContentGO, sceneButtonConfig, this);
            }
        }

        private void CreateConfigMgrBackUI(){
            var backButton = sceneButtonConfigsUIElement.AddButton(250,70,0,0,"Config Manager",SetConfigMgrUIMode);
            backButton.button.onClick.AddListener(delegate{SetConfigMgrUIMode();});
            ApplyHelp(backButton,"Return to the Config Manager screen.");

            var arrow = sceneButtonConfigsUIElement.AddTextBox(">",70,70,250,0,TextAnchor.MiddleCenter);
        }
        private void CreateConfigSetupUI(){
            CreateConfigMgrBackUI();
            
            if (uiMode==UIMode.configRename){
                var configNameLabel = sceneButtonConfigsUIElement.AddTextEntryBox( tempConfigNameJSS,230,70,30,320,0, inputFieldType:InputField.LineType.SingleLine,textEntryEndCallback: RenameConfigEnd);
                ApplyHelp(configNameLabel,"Enter a new name for this Config. Config names must be unique within this plugin setup.");

            }
            else {
                if (UIASceneButtonConfig.configNames.Count==1){
                    var configName = sceneButtonConfigsUIElement.AddTextBox(" "+modifyingConfig.configName,230,70,320,0);
                    ApplyHelp(configName,"This is the name of the Config which is currently being modified.");

                } 
                else {

                    var configNamesPopup = sceneButtonConfigsUIElement.AddPopup(modifyingConfigJSSC,RefreshModifyingConfigNameChoices,230,70,320,0,Mathf.Min(UIASceneButtonConfig.configNames.Count*50,500)+20,50,null,0f);
                    ApplyHelp(configNamesPopup,"Select an alternative Config from the dropdown list to be modified.");
                }
                var renameConfigButton = sceneButtonConfigsUIElement.AddButton(170,70,550,0,"Rename...",RenameConfigStart);
                ApplyHelp(renameConfigButton,"Change the name of the Config which is currently being modified.");
                renameConfigButton.buttonColor = Color.yellow;
            }

            var configAltNamePrefix = sceneButtonConfigsUIElement.AddTextBox("Alernate UIAssist Label:",350,70,0,-100);
            var configAltLabel = sceneButtonConfigsUIElement.AddTextEntryBox(modifyingConfig.configLabelJSS,550f,70,30,350,-100,inputFieldType: UnityEngine.UI.InputField.LineType.SingleLine, placeHolderText:"Enter label...");
            ApplyHelp(configAltLabel,"Enter an alternative label for the Config to be used in UIAssist. Config names must be unique, but its possible to have UIAssist display the same menu name for multiple Configs using this alternative label. If this label is set to blank, then the Config name will be used instead.");

            var addButton = sceneButtonConfigsUIElement.AddButton(70f,70f,955,-100,"+",AddNewButton);
            addButton.buttonText.fontSize = 48;
            addButton.buttonColor = new Color(0,0.7f,0);
            addButton.textColor = Color.white;
            ApplyHelp(addButton,"Add a new Button to the current config.");

            var uiaButtonListContentGO = sceneButtonConfigsUIElement.CreateVScrollableContent(AnchorPresets.topLeft,1050,800,new Vector2(0,-175),10,2);
            foreach (var uiaButton in modifyingConfig.sceneButtons){
                var sceneButtonRow = new SceneButtonRowItem(uiaButtonListContentGO, uiaButton, this);
            }
        }

        private void RenameConfigStart(){
            uiMode = UIMode.configRename;
            tempConfigNameJSS.val = modifyingConfig.configName;
            
            RefreshGroupUI(1);
            tempConfigNameJSS.inputField.ActivateInputField();
        }

        private void RenameConfigEnd(string configName){
            uiMode = UIMode.configSetup;

            if (configName!="" && UIASceneButtonConfig.IsConfigNameUnique(configName)) {
                if (aciveButtonConfigJSSC.val == modifyingConfig.configName) aciveButtonConfigJSSC.valNoCallback = configName;
                modifyingConfig.configName = configName;
                modifyingConfigJSSC.val = configName;                                
            }
            else if (configName != modifyingConfigJSSC.val) SuperController.LogMessage("Config name's must be unique.");

            RefreshGroupUI(1);
            RefreshModifyingConfigNameChoices();
            RefreshActiveConfigNameChoices();
        }


        private void CreateButtonSetupUI(){
            CreateConfigMgrBackUI();

            var backButton = sceneButtonConfigsUIElement.AddButton(250,70,320,0,modifyingConfig.configName,ConfigSetupBackButton);
            backButton.button.onClick.AddListener(delegate{
                uiMode = UIMode.configSetup;
                RefreshGroupUI(1);
                });
            ApplyHelp(backButton,"Return to the Config setup screen.");
            
            var arrow = sceneButtonConfigsUIElement.AddTextBox(">",70,70,320+250,0,TextAnchor.MiddleCenter);

            if (uiMode==UIMode.buttonRename){
                var buttonNameTextBox = sceneButtonConfigsUIElement.AddTextEntryBox( tempButtonNameJSS,230,70,30,390+250,0, inputFieldType:InputField.LineType.SingleLine,textEntryEndCallback: RenameButtonEnd);
                ApplyHelp(buttonNameTextBox,"Enter a new Button name. This is the name of the button as it will be displayed in UIAssist. Button names must be unique within a Config.");

            }
            else {
                if (modifyingConfig.buttonNames.Count==1){
                    var buttonNameLabel = sceneButtonConfigsUIElement.AddTextBox(" "+modifyingButton.buttonName,280,70,390+250,0);
                    ApplyHelp(buttonNameLabel,"This is the name of the currently selected Button as it will be displayed in UIAssist.");

                } 
                else {

                    var buttonNamesPopup = sceneButtonConfigsUIElement.AddPopup(modifyingButtonJSSC,RefreshButtonNameChoices,230,70,390+250,0,Mathf.Min(modifyingConfig.buttonNames.Count*50,500)+20,50,null,0f);
                    ApplyHelp(buttonNamesPopup,"Select an alternative button to modify from the drop down list of button names.");
                }
                var renameButtons = sceneButtonConfigsUIElement.AddButton(170,70,390+250+230,0,"Rename...",RenameButtonStart);
                renameButtons.buttonColor = Color.yellow;
                ApplyHelp(renameButtons,"Add a new Button to the current config.");
            }

            
            var buttonColorPicker = sceneButtonConfigsUIElement.AddColorPicker(modifyingButton.buttonColorJSC,520f,520f,0f,-100,"Button Color");
            ApplyHelp(buttonColorPicker,"Modify the background color of the selected button. This will impact how the button is displayed in UIAssist.");
            var buttonTextColorPicker = sceneButtonConfigsUIElement.AddColorPicker(modifyingButton.buttonTextColorJSC,520f,520f,530,-100,"Text Color");
            ApplyHelp(buttonTextColorPicker,"Modify the text color of the selected button. This will impact how the button is displayed in UIAssist.");
            
            var buttonActions = sceneButtonConfigsUIElement.AddButton(500f,70f,0f,-650,"Setup button actions....",SetupButtonActions);
            ApplyHelp(buttonActions,"Configure the actions that will be triggered when the button is activated in UIAssist.");
        }

        private void RenameButtonStart(){
            uiMode = UIMode.buttonRename;
            tempButtonNameJSS.val = modifyingButton.buttonName;
            
            RefreshGroupUI(1);
            tempButtonNameJSS.inputField.ActivateInputField();
        }

        private void RenameButtonEnd(string buttonName){
            uiMode = UIMode.buttonSetup;

            if (buttonName!="" && modifyingConfig.IsButtonNameUnique(buttonName)) {
                modifyingButton.buttonName = buttonName;
                modifyingButtonJSSC.val = buttonName;                                
            }
            else if (buttonName != modifyingButtonJSSC.val) SuperController.LogMessage("Button name's must be unique.");

            RefreshGroupUI(1);
        }

        public void RefreshActiveConfigNameChoices(){

            aciveButtonConfigJSSC.choices = UIASceneButtonConfig.configNames;
        }

        private void AddNewConfig(){
            UIASceneButtonConfig.AddNewConfig();
            RefreshModifyingConfigNameChoices();
            RefreshActiveConfigNameChoices();
            RefreshGroupUI(1);
        }

        private void AddNewButton(){
            modifyingConfig.AddNewButton();
            RefreshGroupUI(1);
        }

        private void ConfigSetupBackButton(){
            uiMode = UIMode.configSetup;
            RefreshGroupUI(1);
        }

        private void SetupButtonActions(){
            modifyingButton.eventTrigger.OpenPanel();
        }
    }

    public class SceneButtonsConfigRowItem
    {
        private GameObject rowGO;
        private SceneButtonConfigsUI parentUI;

        private GameObject configNameTextBoxGO;

        private UIDynamicButton upButton;
        private UIDynamicButton downButton;
        private UIDynamicButton deleteButton;
        private UIDynamicButton modifyButton;

        private bool pointerContext = false;


        private UIASceneButtonConfig sceneButtonConfig;

        public string sceneButtonConfigName {get{
            return sceneButtonConfig.configName;
        }}

        public SceneButtonsConfigRowItem(GameObject parentGO, UIASceneButtonConfig sceneButtonConfig, SceneButtonConfigsUI parentUI)
        {
            var contentsGO = parentGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;
            rowGO = UI.AddChildGOImage(contentsGO, Color.gray, AnchorPresets.hStretchTop, 1020f, 60f, Vector2.zero);
            GeneralPointerEnterExit contextHelp = rowGO.AddComponent<GeneralPointerEnterExit>();
            contextHelp.Init(RowPointerEnter, RowPointerExit);

            this.sceneButtonConfig = sceneButtonConfig;
            this.parentUI = parentUI;

            CreateUI();
        }

        private void RowPointerEnter(){
            pointerContext = true;

            RefreshVisibility();
        }
        private void RowPointerExit(){
            pointerContext = false;
            RefreshVisibility();
        }

        public void ClearUI()
        {
            ClearUIContents();
            GameObject.Destroy(rowGO);
        }

        private void ClearUIContents()
        {
            if (configNameTextBoxGO != null) GameObject.Destroy(configNameTextBoxGO);
            if (upButton != null) GameObject.Destroy(upButton.gameObject);
            if (downButton != null) GameObject.Destroy(downButton.gameObject);
            if (deleteButton != null) GameObject.Destroy(deleteButton.gameObject);
            if (modifyButton != null) GameObject.Destroy(modifyButton.gameObject);

        }

        private void RefreshUI()
        {
            ClearUIContents();
            CreateUI();
        }

        private void CreateUI()
        {
            configNameTextBoxGO = UI.CreateBasicTextBox(rowGO, sceneButtonConfigName, 800f, 60f, 30, 0f, 0f, AnchorPresets.middleLeft, true, 18, HorizontalWrapMode.Overflow, rescaleText: false);
            parentUI.ApplyHelp(configNameTextBoxGO, "This is the current list of Config Names. In many scenarios, only one Config is needed. However, Scene Creators may want to configure multiple Configs where each Config is a different set of actions that can be triggered from UIAssist buttons. An empty Config (ie. one with no Buttons configured) can be used to disable UIAssist from showing any button actions from this plugin.");

            modifyButton= UI.CreateUIButton(rowGO,60f,60f,">",30,0,0,AnchorPresets.topRight,true,false);
            modifyButton.buttonColor = Color.yellow;
            modifyButton.button.onClick.AddListener(delegate{

                parentUI.SetModifyConfigUIMode(sceneButtonConfig);
            });
            parentUI.ApplyHelp(modifyButton,"Open the Config Setup to modify the Button's contained inside this Config.");

            upButton =UI.CreateUIButton(rowGO,50f,50f,"^",30,-130f,0,AnchorPresets.middleRight,true,false);
            upButton.button.onClick.AddListener(delegate{
                UIASceneButtonConfig.MoveBack(sceneButtonConfig);
                parentUI.RefreshGroupUI(1);
            });
            parentUI.ApplyHelp(upButton,"Move the Button Config up in the list of Configs. The only significance of the Config order is that the first Config is always selected as the Active Config on Scene load");

            downButton =UI.CreateUIButton(rowGO,50f,50f,"v",30,-70f,0,AnchorPresets.middleRight,true,false);
            downButton.button.onClick.AddListener(delegate{
                UIASceneButtonConfig.MoveForward(sceneButtonConfig);
                parentUI.RefreshGroupUI(1);
            });
            parentUI.ApplyHelp(downButton,"Move the Button Config down in the list of Configs. The only significance of the Config order is that the first Config is always selected as the Active Config on Scene load");
        
            deleteButton =UI.CreateUIButton(rowGO,50f,50f,"X",30,-200f,0,AnchorPresets.middleRight,true,false);
            deleteButton.buttonColor = Color.red;
            deleteButton.textColor = Color.white;
            deleteButton.button.onClick.AddListener(delegate{
                if (UIASceneButtonConfig.buttonConfigs.Count==0) return;
                UIASceneButtonConfig.DeleteConfig(sceneButtonConfig);
                parentUI.RefreshModifyingConfigNameChoices();
                parentUI.RefreshGroupUI(1);
                parentUI.RefreshActiveConfigNameChoices();
            });
            parentUI.ApplyHelp(deleteButton,"Delete the config from the list of available configs. All buttons inside the Config and their setup will be immediately deleted and can only be retrieved by loading a previously saved Scene file.");

            RefreshVisibility();

        }

        private void RefreshVisibility()
        {
            upButton.gameObject.SetActive(!UIASceneButtonConfig.IsFirstConfig(sceneButtonConfig) && pointerContext);
            downButton.gameObject.SetActive(!UIASceneButtonConfig.IsLastConfig(sceneButtonConfig) && pointerContext);
            deleteButton.gameObject.SetActive(UIASceneButtonConfig.buttonConfigs.Count!=1 && pointerContext);
        }

    }

    public class SceneButtonRowItem
    {
        private GameObject rowGO;
        private SceneButtonConfigsUI parentUI;

        private GameObject buttonNameTextBoxGO;

        private UIDynamicButton upButton;
        private UIDynamicButton downButton;
        private UIDynamicButton deleteButton;
        private UIDynamicButton modifyButton;

        private bool pointerContext = false;


        private UIASceneButton sceneButton;

        public string sceneButtonName {get{
            return sceneButton.buttonName;
        }}

        public SceneButtonRowItem(GameObject parentGO, UIASceneButton sceneButton, SceneButtonConfigsUI parentUI)
        {
            var contentsGO = parentGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;
            rowGO = UI.AddChildGOImage(contentsGO, Color.gray, AnchorPresets.hStretchTop, 1020f, 60f, Vector2.zero);
            GeneralPointerEnterExit contextHelp = rowGO.AddComponent<GeneralPointerEnterExit>();
            contextHelp.Init(RowPointerEnter, RowPointerExit);



            this.sceneButton = sceneButton;
            this.parentUI = parentUI;

            CreateUI();
        }

        private void RowPointerEnter(){
            pointerContext = true;

            RefreshVisibility();
        }
        private void RowPointerExit(){
            pointerContext = false;
            RefreshVisibility();
        }

        public void ClearUI()
        {
            ClearUIContents();
            GameObject.Destroy(rowGO);
        }

        private void ClearUIContents()
        {
            if (buttonNameTextBoxGO != null) GameObject.Destroy(buttonNameTextBoxGO);
            if (upButton != null) GameObject.Destroy(upButton.gameObject);
            if (downButton != null) GameObject.Destroy(downButton.gameObject);
            if (deleteButton != null) GameObject.Destroy(deleteButton.gameObject);
            if (modifyButton != null) GameObject.Destroy(modifyButton.gameObject);

        }

        private void RefreshUI()
        {
            ClearUIContents();
            CreateUI();
        }

        private void CreateUI()
        {
            buttonNameTextBoxGO = UI.CreateBasicTextBox(rowGO, sceneButtonName, 800f, 60f, 30, 0f, 0f, AnchorPresets.middleLeft, true, 18, HorizontalWrapMode.Overflow, rescaleText: false);
            parentUI.ApplyHelp(buttonNameTextBoxGO, "This is the current list of Button Names. Each button can be configured to trigger a set of actions from within the UIAssist interface.");

            modifyButton= UI.CreateUIButton(rowGO,60f,60f,">",30,0,0,AnchorPresets.topRight,true,false);
            modifyButton.buttonColor = Color.yellow;
            modifyButton.button.onClick.AddListener(delegate{
                parentUI.SetModifyButtonUIMode(sceneButton);
            });
            parentUI.ApplyHelp(modifyButton,"Open the Button Setup screen to modify the Button's name, colors and the actions it will trigger when activated from the UIAssist interface.");

            upButton =UI.CreateUIButton(rowGO,50f,50f,"^",30,-130f,0,AnchorPresets.middleRight,true,false);
            upButton.button.onClick.AddListener(delegate{
                parentUI.modifyingConfig.MoveBack(sceneButton);
                parentUI.RefreshGroupUI(1);
            });
            parentUI.ApplyHelp(upButton,"Move the button up in the list order. This will impact the order that the buttons are displayed in UIAssist.");

            downButton =UI.CreateUIButton(rowGO,50f,50f,"v",30,-70f,0,AnchorPresets.middleRight,true,false);
            downButton.button.onClick.AddListener(delegate{
                parentUI.modifyingConfig.MoveForward(sceneButton);
                parentUI.RefreshGroupUI(1);
            });
            parentUI.ApplyHelp(downButton,"Move the button down in the list order. This will impact the order that the buttons are displayed in UIAssist.");
        
            deleteButton =UI.CreateUIButton(rowGO,50f,50f,"X",30,-200f,0,AnchorPresets.middleRight,true,false);
            deleteButton.buttonColor = Color.red;
            deleteButton.textColor = Color.white;
            deleteButton.button.onClick.AddListener(delegate{
                if (UIASceneButtonConfig.buttonConfigs.Count==0) return;
                parentUI.modifyingConfig.DeleteButton(sceneButton);
                parentUI.RefreshGroupUI(1);
            });
            parentUI.ApplyHelp(deleteButton,"Delete the Button from this Config. The button and its setup will be immediately deleted.");

            RefreshVisibility();

        }

        private void RefreshVisibility()
        {
            upButton.gameObject.SetActive(!parentUI.modifyingConfig.IsFirstButton(sceneButton) && pointerContext);
            downButton.gameObject.SetActive(!parentUI.modifyingConfig.IsLastButton(sceneButton) && pointerContext);
            deleteButton.gameObject.SetActive( pointerContext);
        }

    }

}