﻿using SimpleJSON;
using System.Collections.Generic;
using UnityEngine.EventSystems;
using System.Linq;
using Battlehub.RTCommon;
using UnityEngine;

namespace JayJayWon
{
    public class UIASceneButtonConfig
    {
        private static List<UIASceneButtonConfig> _buttonConfigs = new List<UIASceneButtonConfig>();

        public static List<UIASceneButtonConfig> buttonConfigs
        {
            get
            {
                return _buttonConfigs;
            }
        }

        public static List<string> configNames
        {
            get
            {
                return _buttonConfigs.Select(x => x.configName).ToList();
            }
        }

        public static UIASceneButtonConfig GetConfig(string configName){
            if (!configNames.Contains(configName)) return null;

            return buttonConfigs.Where(x => x.configName == configName).FirstOrDefault();
        }

        public static UIASceneButtonConfig AddNewConfig(){
            int configRef = 1;

            string configName ="";
            bool uniqueConfigFound = false;

            while(!uniqueConfigFound){
                configName = "Config "+configRef.ToString();
                if (IsConfigNameUnique( configName)) uniqueConfigFound = true;
                else configRef++;
            }

            var newConfig = new UIASceneButtonConfig(configName);
            _buttonConfigs.Add(newConfig);

            return newConfig;

        }

        public static void MoveBack(UIASceneButtonConfig sceneButtonConfig){
            if (IsFirstConfig(sceneButtonConfig)) return;

            int index = _buttonConfigs.IndexOf(sceneButtonConfig);
            _buttonConfigs.Remove(sceneButtonConfig);
            _buttonConfigs.Insert(index-1,sceneButtonConfig);

        }

        public static void MoveForward(UIASceneButtonConfig sceneButtonConfig){
            if (IsLastConfig(sceneButtonConfig)) return;

            int index = _buttonConfigs.IndexOf(sceneButtonConfig);
            _buttonConfigs.Remove(sceneButtonConfig);
            _buttonConfigs.Insert(index+1,sceneButtonConfig);
        }

        public static bool IsFirstConfig(UIASceneButtonConfig sceneButtonConfig)
        {
            return _buttonConfigs.IndexOf(sceneButtonConfig)==0;
        }

        public static bool IsLastConfig(UIASceneButtonConfig sceneButtonConfig)
        {
            return _buttonConfigs.IndexOf(sceneButtonConfig)==_buttonConfigs.Count-1;
        }

        public static void DeleteConfig(UIASceneButtonConfig sceneButtonConfig)
        {
            if (_buttonConfigs.Contains(sceneButtonConfig)) _buttonConfigs.Remove(sceneButtonConfig);
        }

        public static bool IsConfigNameUnique(string configName)
        {
            return !configNames.Contains(configName);
        }

        public static JSONClass GetJSON()
        {
            var jc = new JSONClass();

            var buttonConfigsJA = new JSONArray();

            foreach (var buttonConfig in _buttonConfigs)
            {
                buttonConfigsJA.Add(buttonConfig.GetJSONInternal());
            }
            jc["buttonConfigsList"] = buttonConfigsJA;

            return jc;
        }

        public static void RestoreFromJSON(JSONClass jc)
        {
            _buttonConfigs.Clear();

            foreach (JSONClass buttonConfigJC in jc["buttonConfigsList"].AsArray)
            {
                var buttonConfig = new UIASceneButtonConfig("");
                

                buttonConfig.RestoreFromJSONInternal(buttonConfigJC);
                if (IsConfigNameUnique(buttonConfig.configName)) _buttonConfigs.Add(buttonConfig);
                else SuperController.LogMessage("UIASceneButtonConfig.RestoreFromJSON: Duplicate Button Config name " + buttonConfig.configName+ " detected. Ignoring duplicate.");

            }
        }


        public JSONStorableString configNameJSS;
        public JSONStorableString configLabelJSS;

        public string configName { get { return configNameJSS.val; } set { configNameJSS.val = value; } }
        public string configLabel { get { return configLabelJSS.val; } set { configLabelJSS.val = value; } }

        private List<UIASceneButton> _sceneButtons = new List<UIASceneButton>();

        public UIASceneButtonConfig(string configName)
        {
            configNameJSS = new JSONStorableString("configName", "",ConfigNameChangeCallback);
            configNameJSS.valNoCallback = configName;
            configLabelJSS = new JSONStorableString("configLabel", "", ConfigLabelChangeCallback);
        }

        private void ConfigNameChangeCallback(string configName){
            if (configLabel=="") UIASceneButtonsSetup.RefreshIfCurrentUIA(this);
        }

        private void ConfigLabelChangeCallback(string configLabel){
            UIASceneButtonsSetup.RefreshIfCurrentUIA(this);

        }

        public UIASceneButton GetButton(string buttonName){
            return _sceneButtons.Where(x=>x.buttonName==buttonName).First();
        }

        public List<string> buttonNames
        {
            get
            {
                return _sceneButtons.Select(x => x.buttonName).ToList();
            }
        }

        public List<UIASceneButton> sceneButtons { get{ return _sceneButtons;}}

        public UIASceneButton AddNewButton(){
            int buttonRef = 1;

            string buttonName ="";
            bool uniqueButtonNameFound = false;

            while(!uniqueButtonNameFound){
                buttonName = "Button "+buttonRef.ToString();
                if (IsButtonNameUnique( buttonName)) uniqueButtonNameFound = true;
                else buttonRef++;
            }

            var newButton = new UIASceneButton(buttonName,this);
            _sceneButtons.Add(newButton);

            UIASceneButtonsSetup.RefreshIfCurrentUIA(this);

            return newButton;

        }

        public void MoveBack(UIASceneButton sceneButton){
            if (IsFirstButton(sceneButton)) return;

            int index = _sceneButtons.IndexOf(sceneButton);
            _sceneButtons.Remove(sceneButton);
            _sceneButtons.Insert(index-1,sceneButton);

            UIASceneButtonsSetup.RefreshIfCurrentUIA(this);

        }

        public void MoveForward(UIASceneButton sceneButton){
            if (IsLastButton(sceneButton)) return;
            
            int index = _sceneButtons.IndexOf(sceneButton);
            _sceneButtons.Remove(sceneButton);
            _sceneButtons.Insert(index+1,sceneButton);

            UIASceneButtonsSetup.RefreshIfCurrentUIA(this);
        }

        public bool IsFirstButton(UIASceneButton sceneButton)
        {
            return _sceneButtons.IndexOf(sceneButton)==0;
        }

        public bool IsLastButton(UIASceneButton sceneButton)
        {
            return _sceneButtons.IndexOf(sceneButton)==_sceneButtons.Count-1;
        }

        public void DeleteButton(UIASceneButton sceneButton)
        {
            if (_sceneButtons.Contains(sceneButton)) _sceneButtons.Remove(sceneButton);

            UIASceneButtonsSetup.RefreshIfCurrentUIA(this);
        }

        public bool IsButtonNameUnique(string buttonName)
        {
            return !buttonNames.Contains(buttonName);
        }



        private JSONClass GetJSONInternal()
        {
            var jc = new JSONClass();
            configNameJSS.StoreJSON(jc);
            configLabelJSS.StoreJSON(jc);

            var buttonsJA = new JSONArray();
            foreach (var sceneButton in _sceneButtons)
            {
                buttonsJA.Add(sceneButton.GetJSON());
            }
            jc["buttons"] = buttonsJA;

            return jc;
        }

        private void RestoreFromJSONInternal(JSONClass jc)
        {
            
            configNameJSS.RestoreFromJSON(jc);
            configLabelJSS.RestoreFromJSON(jc);

            _sceneButtons.Clear();

            foreach (JSONClass buttonJC in jc["buttons"].AsArray)
            {
                var uiaSceneButton = new UIASceneButton("", this);
                uiaSceneButton.RestoreFromJSON(buttonJC);

                if (IsButtonNameUnique(uiaSceneButton.buttonName)) _sceneButtons.Add(uiaSceneButton);
                else SuperController.LogMessage("UIASceneButtonConfig.RestoreFromJSONInternal: Duplicate Button Label " + uiaSceneButton.buttonName + " detected. Ignoring duplicate.");
            }
        }
    }

    public class UIASceneButton
    {
        public JSONStorableString buttonLabelJSS;
        public JSONStorableColor buttonColorJSC;
        public JSONStorableColor buttonTextColorJSC;

        public string buttonName { get { return buttonLabelJSS.val; } set { buttonLabelJSS.val = value; } }

        public EventTrigger eventTrigger = new EventTrigger(UIASceneButtonsSetup.mvrScript,"buttonActions");

        private UIASceneButtonConfig parentConfig ;

        public UIASceneButton(string label, UIASceneButtonConfig parentConfig)
        {
            buttonLabelJSS = new JSONStorableString("buttonLabel", "",ButtonLabelChangeCallback);
            buttonLabelJSS.valNoCallback = label;

            var buttonColor = new Color(0.75f,0.75f,0.75f) ;
            buttonColorJSC = new JSONStorableColor("buttonColor", UIASceneButtonsSetup.RGBToHSV(buttonColor),ButtonColorChangeCallback);
            buttonTextColorJSC = new JSONStorableColor("buttonTextColor", new HSVColor(),ButtonColorChangeCallback);
            this.parentConfig = parentConfig;
        }

        private void ButtonLabelChangeCallback(string label){
            UIASceneButtonsSetup.RefreshIfCurrentUIA(parentConfig);
        }

        private void ButtonColorChangeCallback(JSONStorableColor colorJSC){
            UIASceneButtonsSetup.RefreshIfCurrentUIA(parentConfig);
        }

        public JSONClass GetJSON()
        {
            var jc = new JSONClass();

            buttonLabelJSS.StoreJSON(jc);
            buttonColorJSC.StoreJSON(jc);
            buttonTextColorJSC.StoreJSON(jc);

            jc["eventTrigger"] = eventTrigger.GetJSON();

            return jc;
        }

        public void RestoreFromJSON(JSONClass jc)
        {

            buttonLabelJSS.RestoreFromJSON(jc);
            buttonColorJSC.RestoreFromJSON(jc);
            buttonTextColorJSC.RestoreFromJSON(jc);

            eventTrigger.RestoreFromJSON(jc["eventTrigger"].AsObject);
        }
    }

}
