using AssetBundles;
using SimpleJSON;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.EventSystems;
using UnityEngine.XR;
using System.Linq;

namespace JayJayWon
{
    public class UIASceneButtonsSetup : MVRScript
    {

        public static MVRScript mvrScript;
        public static bool isVREnabled = false;

        private static JSONStorableString uiaButtonPressCallbackJSS = new JSONStorableString("uiaButtonPressCallback", "",UIAButtonPressCallback);
        public static JSONStorableAction forceButtonListDataRefreshJSA = new JSONStorableAction("forceButtonListDataRefresh", null);

        public static UIASceneButtonConfig currentlyRegisteredConfig = null;

        private SceneButtonConfigsUI sceneButtonConfigsUI;

        public static JSONStorableStringChooser activeButtonConfigJSSC {get;private set;}

        private bool scenePlugin = true;

        public override void Init()
        {
            if (containingAtom.name == "CoreControl" && containingAtom.type == "SessionPluginManager") scenePlugin=false;

            if (scenePlugin){
                if (XRSettings.enabled) isVREnabled = true;
                else isVREnabled = false;
                SimpleTriggerHandler.LoadAssets();
                mvrScript = this;     

                activeButtonConfigJSSC = new JSONStorableStringChooser("ActiveConfig",new List<string>(),"","Active UIA Button Config", RefreshUIA);
                activeButtonConfigJSSC.isRestorable = false;
                activeButtonConfigJSSC.isStorable = false;
                RegisterStringChooser(activeButtonConfigJSSC);
            }



        }

        public void OnDestroy()
        {
            // Unregisters this plugin with other plugins using interop spec
            if (scenePlugin) SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
        }
        public void Start()
        {
            if (scenePlugin){
                if (UIASceneButtonConfig.buttonConfigs.Count==0) UIASceneButtonConfig.AddNewConfig();
                activeButtonConfigJSSC.val = UIASceneButtonConfig.configNames[0];
               

                // Registers this plugin with other plugins using interop spec
                SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);

                sceneButtonConfigsUI = new SceneButtonConfigsUI();
                sceneButtonConfigsUI.SetModifyConfigUIMode(UIASceneButtonConfig.GetConfig(activeButtonConfigJSSC.val));
                sceneButtonConfigsUI.RefreshGroupUI(1);
                sceneButtonConfigsUI.RefreshActiveConfigNameChoices();
                sceneButtonConfigsUI.RefreshModifyingConfigNameChoices();
            }
            else{
                CreateTextField(new JSONStorableString("warning","This plugin should not be loaded as a Session plugin."));
            }

        }

        private static void UIAButtonPressCallback(string buttonName){
            currentlyRegisteredConfig.GetButton(buttonName).eventTrigger.active = true;
            uiaButtonPressCallbackJSS.valNoCallback="";
            currentlyRegisteredConfig.GetButton(buttonName).eventTrigger.active = false;
        }

        private static void RefreshUIA(string configName){            

            if (forceButtonListDataRefreshJSA.actionCallback !=null) forceButtonListDataRefreshJSA.actionCallback.Invoke();
        }

        public static void RefreshIfCurrentUIA(UIASceneButtonConfig sceneButtonConfig){
            if (sceneButtonConfig==currentlyRegisteredConfig) RefreshUIA(currentlyRegisteredConfig.configName);
        }

        public void OnUIAButtonListDataRequested(Dictionary<string, object> buttonListData)
        {
            // This is included for future proofing, in case there is a need to radically change the integration spec in the future
            buttonListData.Add("IntegrationVersion", 1);

            currentlyRegisteredConfig = UIASceneButtonConfig.buttonConfigs.Where(x => x.configName==activeButtonConfigJSSC.val).FirstOrDefault();

            // This identifies this source plugin, in case mutiple plugins attempt to register with UIAssist.
            var label = currentlyRegisteredConfig.configLabel;
            if (label=="") label = currentlyRegisteredConfig.configName;
            buttonListData.Add("ButtonListProviderName", label);

            var buttonNameList = new List<string>() ;
            var buttonColors = new Dictionary<string, Color>();
            var buttonTextColors = new Dictionary<string, Color>();

            foreach (var sceneButton in currentlyRegisteredConfig.sceneButtons) {
                buttonNameList.Add(sceneButton.buttonName);
                buttonColors.Add(sceneButton.buttonName, HSVToRGB(sceneButton.buttonColorJSC.val) );
                buttonTextColors.Add(sceneButton.buttonName, HSVToRGB(sceneButton.buttonTextColorJSC.val) );
            }

            // This is the list of button names to be passed to UIAssist. Each list entry will be presented as a button in UIAssist.
            buttonListData.Add("ButtonNameList", buttonNameList);
            buttonListData.Add("ButtonColorsByName", buttonColors);
            buttonListData.Add("ButtonTextColorsByName", buttonTextColors);

            // This is the JSONStorable String that UIAssist will modify when one of the buttons is pressed. A callback should be set on this JSONStorableString to respond to when one of the buttons is pressed in UIAssist. The value of the JSONStorableString will allow this plugin to determine which button has been pressed in UIAssist.
            buttonListData.Add("ButtonPressCallbackJSS", uiaButtonPressCallbackJSS);

            // This is a JSONStorableAction that UIAssist will populate with a callback. When the action is invoked (from this plugin), then UIAssist will call the OnUIAButtonListDateRequested method again to obtain an updated version of the list of buttons.
            buttonListData.Add("ForceButtonListDataRefresh", forceButtonListDataRefreshJSA);

        }


        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            
            var jc = base.GetJSON(includePhysical, includeAppearance, true);
            jc["buttonConfigs"] = UIASceneButtonConfig.GetJSON();
            return jc;
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            
            UIASceneButtonConfig.RestoreFromJSON(jc["buttonConfigs"].AsObject);
            base.RestoreFromJSON(jc,restorePhysical,restoreAppearance,presetAtoms,setMissingToDefault);

        }

        public static Color HSVToRGB(HSVColor hSVColor){
            return Color.HSVToRGB(hSVColor.H,hSVColor.S,hSVColor.V);
        }

        public static HSVColor RGBToHSV(Color color){
            HSVColor hsvColor = new HSVColor();
            Color.RGBToHSV(color, out hsvColor.H, out hsvColor.S,out hsvColor.V);
            return hsvColor;
        }
    }


}

    
